@@ -24,21 +24,21 @@
 
 package net.sf.picard.analysis;
 
-import net.sf.picard.reference.ReferenceSequence;
-import net.sf.picard.util.RExecutor;
 import net.sf.picard.PicardException;
-import net.sf.picard.cmdline.CommandLineProgram;
 import net.sf.picard.cmdline.Option;
-import net.sf.picard.cmdline.StandardOptionDefinitions;
 import net.sf.picard.io.IoUtil;
 import net.sf.picard.metrics.MetricsFile;
+import net.sf.picard.reference.ReferenceSequence;
 import net.sf.picard.util.Histogram;
+import net.sf.picard.util.Log;
+import net.sf.picard.util.RExecutor;
 import net.sf.samtools.SAMFileHeader;
-import net.sf.samtools.SAMFileReader;
+import net.sf.samtools.SAMReadGroupRecord;
 import net.sf.samtools.SAMRecord;
 
 import java.io.File;
 import java.util.Arrays;
+import java.util.List;
 
 
 /**
@@ -49,18 +49,30 @@ import java.util.Arrays;
  * @author Tim Fennell
  */
 public class MeanQualityByCycle extends SinglePassSamProgram {
-    @Option(shortName="CHART", doc="A file (with .pdf extension) to write the chart to")
+
+	public final String USAGE = getStandardUsagePreamble() + "Program to generate a data table and chart of " +
+			"mean quality by cycle from a SAM or BAM file.  Works best on a single lane/run of data, but can be applied to" +
+			"merged BAMs.";
+
+	@Option(shortName="CHART", doc="A file (with .pdf extension) to write the chart to.")
     public File CHART_OUTPUT;
 
-    @Option(doc="If set to true calculate mean quality over aligned reads only")
+    @Option(doc="If set to true, calculate mean quality over aligned reads only.")
     public boolean ALIGNED_READS_ONLY = false;
 
-    @Option(doc="If set to true calculate mean quality over PF reads only")
+    @Option(doc="If set to true calculate mean quality over PF reads only.")
     public boolean PF_READS_ONLY = false;
 
     private HistogramGenerator q  = new HistogramGenerator(false);
     private HistogramGenerator oq = new HistogramGenerator(true);
 
+    /**
+     * A subtitle for the plot, usually corresponding to a library.
+     */
+    private String plotSubtitle = "";
+
+    private final Log log = Log.getInstance(MeanQualityByCycle.class);
+
     /** Required main method. */
     public static void main(String[] args) {
         System.exit(new MeanQualityByCycle().instanceMain(args));
@@ -125,10 +137,9 @@ public class MeanQualityByCycle extends SinglePassSamProgram {
             }
 
             for (int i=0; i< secondReadTotalsByCycle.length; ++i) {
-                final int cycle = firstReadLength + i;
-
                 if (secondReadCountsByCycle[i] > 0) {
-                    meanQualities.increment(cycle, secondReadTotalsByCycle[i] / firstReadCountsByCycle[i]);
+                    final int cycle = firstReadLength + i;
+                    meanQualities.increment(cycle, secondReadTotalsByCycle[i] / secondReadCountsByCycle[i]);
                 }
             }
 
@@ -144,6 +155,13 @@ public class MeanQualityByCycle extends SinglePassSamProgram {
     @Override
     protected void setup(final SAMFileHeader header, final File samFile) {
         IoUtil.assertFileIsWritable(CHART_OUTPUT);
+        // If we're working with a single library, assign that library's name
+        // as a suffix to the plot title
+        final List<SAMReadGroupRecord> readGroups = header.getReadGroups();
+        if (readGroups.size() == 1) {
+            this.plotSubtitle = readGroups.get(0).getLibrary();
+            if (null == this.plotSubtitle) this.plotSubtitle = "";
+        }
     }
 
     @Override
@@ -165,15 +183,21 @@ public class MeanQualityByCycle extends SinglePassSamProgram {
         if (!oq.isEmpty()) metrics.addHistogram(oq.getMeanQualityHistogram());
         metrics.write(OUTPUT);
 
-        // Now run R to generate a chart
-        final int rResult = RExecutor.executeFromClasspath(
-                "net/sf/picard/analysis/meanQualityByCycle.R",
-                OUTPUT.getAbsolutePath(),
-                CHART_OUTPUT.getAbsolutePath(),
-                INPUT.getName());
+        if (q.isEmpty() && oq.isEmpty()) {
+            log.warn("No valid bases found in input file. No plot will be produced.");
+        }
+        else {
+            // Now run R to generate a chart
+            final int rResult = RExecutor.executeFromClasspath(
+                    "net/sf/picard/analysis/meanQualityByCycle.R",
+                    OUTPUT.getAbsolutePath(),
+                    CHART_OUTPUT.getAbsolutePath(),
+                    INPUT.getName(),
+                    this.plotSubtitle);
 
-        if (rResult != 0) {
-            throw new PicardException("R script meanQualityByCycle.R failed with return code " + rResult);
+            if (rResult != 0) {
+                throw new PicardException("R script meanQualityByCycle.R failed with return code " + rResult);
+            }
         }
     }
 }
