@@ -24,6 +24,7 @@
 package net.sf.picard.cmdline;
 
 import java.io.File;
+import java.net.InetAddress;
 import java.text.DecimalFormat;
 import java.util.*;
 
@@ -32,11 +33,13 @@ import net.sf.picard.metrics.MetricBase;
 import net.sf.picard.metrics.MetricsFile;
 import net.sf.picard.metrics.StringHeader;
 import net.sf.picard.util.Log;
+import net.sf.samtools.Defaults;
 import net.sf.samtools.SAMFileReader;
 import net.sf.samtools.SAMFileWriterFactory;
 import net.sf.samtools.SAMFileWriterImpl;
 import net.sf.samtools.util.BlockCompressedOutputStream;
 import net.sf.samtools.util.BlockCompressedStreamConstants;
+import net.sf.samtools.util.IOUtil;
 
 /**
  * Abstract class to facilitate writing command-line programs.
@@ -49,7 +52,7 @@ import net.sf.samtools.util.BlockCompressedStreamConstants;
  * 2. If there is any custom command-line validation, override customCommandLineValidation().  When this method is
  * called, the command line has been parsed and set into the data members of the concrete class.
  *
- * 3. Implement a method doWork().  This is called after successful comand-line processing.  The value it returns is
+ * 3. Implement a method doWork().  This is called after successful command-line processing.  The value it returns is
  * the exit status of the program.  It is assumed that the concrete class emits any appropriate error message before
  * returning non-zero.  doWork() may throw unchecked exceptions, which are caught and reported appropriately.
  *
@@ -63,10 +66,8 @@ import net.sf.samtools.util.BlockCompressedStreamConstants;
  */
 public abstract class CommandLineProgram {
 
-    @Option(common=true)
-    public File TMP_DIR = (System.getProperty("java.io.tmpdir").endsWith("/" + System.getProperty("user.name"))?
-            new File(System.getProperty("java.io.tmpdir")):
-            new File(System.getProperty("java.io.tmpdir"), System.getProperty("user.name")));
+    @Option(common=true, optional=true)
+    public List<File> TMP_DIR = new ArrayList<File>();
 
     @Option(doc = "Control verbosity of logging.", common=true)
     public Log.LogLevel VERBOSITY = Log.LogLevel.INFO;
@@ -86,10 +87,10 @@ public abstract class CommandLineProgram {
     public Integer MAX_RECORDS_IN_RAM = SAMFileWriterImpl.getDefaultMaxRecordsInRam();
 
     @Option(doc = "Whether to create a BAM index when writing a coordinate-sorted BAM file.", common=true)
-    public Boolean CREATE_INDEX = false;
+    public Boolean CREATE_INDEX = Defaults.CREATE_INDEX;
 
     @Option(doc="Whether to create an MD5 digest for any BAM files created.  ", common=true)
-    public boolean CREATE_MD5_FILE = false;
+    public boolean CREATE_MD5_FILE = Defaults.CREATE_MD5;
 
     private final String standardUsagePreamble = CommandLineParser.getStandardUsagePreamble(getClass());
 
@@ -123,13 +124,17 @@ public abstract class CommandLineProgram {
             return 1;
         }
 
+        // Provide one temp directory if the caller didn't
+        if (this.TMP_DIR == null) this.TMP_DIR = new ArrayList<File>();
+        if (this.TMP_DIR.isEmpty()) TMP_DIR.add(IOUtil.getDefaultTmpDir());
+
         // Build the default headers
         final Date startDate = new Date();
         this.defaultHeaders.add(new StringHeader(commandLine));
         this.defaultHeaders.add(new StringHeader("Started on: " + startDate));
 
         Log.setGlobalLogLevel(VERBOSITY);
-        SAMFileReader.ValidationStringency originalStringency = SAMFileReader.getDefaultValidationStringency();
+        final SAMFileReader.ValidationStringency originalStringency = SAMFileReader.getDefaultValidationStringency();
         SAMFileReader.setDefaultValidationStringency(VALIDATION_STRINGENCY);
         BlockCompressedOutputStream.setDefaultCompressionLevel(COMPRESSION_LEVEL);
 
@@ -143,17 +148,31 @@ public abstract class CommandLineProgram {
 
         SAMFileWriterFactory.setDefaultCreateMd5File(CREATE_MD5_FILE);
 
-        if (!TMP_DIR.exists()) {
-            // Intentially not checking the return value, because it may be that the program does not
+        for (final File f : TMP_DIR) {
+            // Intentially not checking the return values, because it may be that the program does not
             // need a tmp_dir. If this fails, the problem will be discovered downstream.
-            TMP_DIR.mkdirs();
+            if (!f.exists()) f.mkdirs();
+            f.setReadable(true, false);
+            f.setWritable(true, false);
+            System.setProperty("java.io.tmpdir", f.getAbsolutePath()); // in loop so that last one takes effect
         }
-        System.setProperty("java.io.tmpdir", TMP_DIR.getAbsolutePath());
+
         if (!QUIET) {
             System.err.println("[" + new Date() + "] " + commandLine);
-        }
-        final int ret;
 
+            // Output a one liner about who/where and what software/os we're running on
+            try {
+            System.err.println("[" + new Date() + "] Executing as " +
+                                       System.getProperty("user.name") + "@" + InetAddress.getLocalHost().getHostName() +
+                                       " on " + System.getProperty("os.name") + " " + System.getProperty("os.version") +
+                                       " " + System.getProperty("os.arch") + "; " + System.getProperty("java.vm.name") +
+                                       " " + System.getProperty("java.runtime.version") +
+                    "; Picard version: " + commandLineParser.getVersion());
+            }
+            catch (Exception e) { /* Unpossible! */ }
+        }
+
+        int ret = -1;
         try {
             ret = doWork();
         } finally {
@@ -166,6 +185,7 @@ public abstract class CommandLineProgram {
                     final String elapsedString  = new DecimalFormat("#,##0.00").format(elapsedMinutes);
                     System.err.println("[" + endDate + "] " + getClass().getName() + " done. Elapsed time: " + elapsedString + " minutes.");
                     System.err.println("Runtime.totalMemory()=" + Runtime.getRuntime().totalMemory());
+                    if (ret != 0 && CommandLineParser.hasWebDocumentation(this.getClass())) System.err.println(CommandLineParser.getFaqLink());
                 }
             }
             catch (Throwable e) {
@@ -183,6 +203,21 @@ public abstract class CommandLineProgram {
     * to be written to the appropriate place.
     */
     protected String[] customCommandLineValidation() {
+        final List<String> ret = new ArrayList<String>();
+        for (final Object childOptionsObject : getNestedOptions().values()) {
+            if (childOptionsObject instanceof CommandLineProgram) {
+                final CommandLineProgram childClp = (CommandLineProgram)childOptionsObject;
+                final String[] childErrors = childClp.customCommandLineValidation();
+                if (childErrors != null) {
+                    for (final String error: childErrors) {
+                        ret.add(error);
+                    }
+                }
+            }
+        }
+        if (!ret.isEmpty()) {
+            ret.toArray(new String[ret.size()]);
+        }
         return null;
     }
 
@@ -227,15 +262,26 @@ public abstract class CommandLineProgram {
         return commandLineParser;
     }
 
+    /**
+     * @return This is a little-used version string that can be put in the @Usage annotation
+     */
     public String getProgramVersion() {
         return commandLineParser.getProgramVersion();
     }
 
+
+    /**
+     * @return Version stored in the manifest of the jarfile.
+     */
+    public String getVersion() {
+        return getCommandLineParser().getVersion();
+    }
+
     public String getCommandLine() {
         return commandLine;
     }
 
-    public void setDefaultHeaders(List<Header> headers) {
+    public void setDefaultHeaders(final List<Header> headers) {
         this.defaultHeaders.clear();
         this.defaultHeaders.addAll(headers);
     }
@@ -243,5 +289,22 @@ public abstract class CommandLineProgram {
     public List<Header> getDefaultHeaders() {
         return this.defaultHeaders;
     }
-}
 
+    /**
+     * @return Map of nested options, where the key is the prefix to be used when specifying Options inside of a nested
+     * options object, and the value is the nested options object itself.  Default implementation is to return a
+     * map of all the fields annotated with @NestedOptions, with key being the field name.
+     */
+    public Map<String, Object> getNestedOptions() {
+        return CommandLineParser.getNestedOptions(this);
+    }
+
+    /**
+     * @return Map of nested options, where the key is the prefix to be used when specifying Options inside of a nested
+     * options object, and the value is the nested options object itself, for the purpose of generating help.
+     * Default implementation is to return the same map as getNestedOptions().
+     */
+    public Map<String, Object> getNestedOptionsForHelp() {
+        return getNestedOptions();
+    }
+}
