@@ -30,11 +30,15 @@ import net.sf.picard.reference.ReferenceSequenceFileWalker;
 import net.sf.picard.util.CigarUtil;
 import net.sf.picard.util.Log;
 import net.sf.samtools.*;
+import net.sf.samtools.SAMFileHeader.SortOrder;
 import net.sf.samtools.util.CloseableIterator;
 import net.sf.samtools.util.SequenceUtil;
 import net.sf.samtools.util.SortingCollection;
+import net.sf.picard.filter.FilteringIterator;
 
 import java.io.File;
+import java.text.DecimalFormat;
+import java.text.NumberFormat;
 import java.util.ArrayList;
 import java.util.List;
 
@@ -65,11 +69,12 @@ public abstract class AbstractAlignmentMerger {
     public static final int MAX_RECORDS_IN_RAM = 500000;
 
     private static final char[] RESERVED_ATTRIBUTE_STARTS = {'X','Y', 'Z'};
+    private final NumberFormat FMT = new DecimalFormat("#,###");
 
     private final Log log = Log.getInstance(AbstractAlignmentMerger.class);
     private final File unmappedBamFile;
     private final File targetBamFile;
-    private SAMSequenceDictionary sequenceDictionary = null;
+    private final SAMSequenceDictionary sequenceDictionary;
     private ReferenceSequenceFileWalker refSeq = null;
     private final boolean clipAdapters;
     private final boolean bisulfiteSequence;
@@ -81,16 +86,26 @@ public abstract class AbstractAlignmentMerger {
     private final Integer read1BasesTrimmed;
     private final Integer read2BasesTrimmed;
     private final List<SamPairUtil.PairOrientation> expectedOrientations;
+    private final SortOrder sortOrder;
     private MultiHitAlignedReadIterator alignedIterator = null;
-    private SamRecordFilter alignmentFilter = new SamRecordFilter() {
-        public boolean filterOut(SAMRecord record) {
+    private boolean clipOverlappingReads = true;
+    private int maxRecordsInRam = MAX_RECORDS_IN_RAM;
+    private final PrimaryAlignmentSelectionStrategy primaryAlignmentSelectionStrategy;
+    private boolean keepAlignerProperPairFlags = false;
+
+    private final SamRecordFilter alignmentFilter = new SamRecordFilter() {
+        public boolean filterOut(final SAMRecord record) {
             return ignoreAlignment(record);
         }
+        public boolean filterOut(final SAMRecord first, final SAMRecord second) {
+            throw new UnsupportedOperationException("Paired SamRecordFilter not implemented!");
+        }
     };
+    private boolean includeSecondaryAlignments = true;
 
     protected abstract CloseableIterator<SAMRecord> getQuerynameSortedAlignedRecords();
     
-    protected boolean ignoreAlignment(SAMRecord sam) { return false; } // default implementation
+    protected boolean ignoreAlignment(final SAMRecord sam) { return false; } // default implementation
 
     /**
      * Constructor
@@ -113,13 +128,19 @@ public abstract class AbstractAlignmentMerger {
      * @param read2BasesTrimmed The number of bases trimmed from start of read 2 prior to alignment.  Optional.
      * @param expectedOrientations A List of SamPairUtil.PairOrientations that are expected for
      *                          aligned pairs.  Used to determine the properPair flag.
+     * @param sortOrder           The order in which the merged records should be output.  If null,
+     *                            output will be coordinate-sorted
+     * @param primaryAlignmentSelectionStrategy What to do when there are multiple primary alignments, or multiple
+     *                                          alignments but none primary, for a read or read pair.
      */
     public AbstractAlignmentMerger(final File unmappedBamFile, final File targetBamFile,
                                    final File referenceFasta, final boolean clipAdapters,
                                    final boolean bisulfiteSequence, final boolean alignedReadsOnly,
                                    final SAMProgramRecord programRecord, final List<String> attributesToRetain,
                                    final Integer read1BasesTrimmed, final Integer read2BasesTrimmed,
-                                   final List<SamPairUtil.PairOrientation> expectedOrientations) {
+                                   final List<SamPairUtil.PairOrientation> expectedOrientations,
+                                   final SAMFileHeader.SortOrder sortOrder,
+                                   final PrimaryAlignmentSelectionStrategy primaryAlignmentSelectionStrategy) {
         IoUtil.assertFileIsReadable(unmappedBamFile);
         IoUtil.assertFileIsWritable(targetBamFile);
         IoUtil.assertFileIsReadable(referenceFasta);
@@ -130,13 +151,18 @@ public abstract class AbstractAlignmentMerger {
 
         this.refSeq = new ReferenceSequenceFileWalker(referenceFasta);
         this.sequenceDictionary = refSeq.getSequenceDictionary();
+        if (this.sequenceDictionary == null) {
+            throw new PicardException("No sequence dictionary found for " + referenceFasta.getAbsolutePath() +
+                    ".  Use CreateSequenceDictionary.jar to create a sequence dictionary.");
+        }
 
         this.clipAdapters = clipAdapters;
         this.bisulfiteSequence = bisulfiteSequence;
         this.alignedReadsOnly = alignedReadsOnly;
 
         this.header = new SAMFileHeader();
-        header.setSortOrder(SAMFileHeader.SortOrder.coordinate);
+        this.sortOrder = sortOrder != null ? sortOrder : SortOrder.coordinate;
+        header.setSortOrder(SortOrder.coordinate);
         if (programRecord != null) {
             setProgramRecord(programRecord);
         }
@@ -148,13 +174,32 @@ public abstract class AbstractAlignmentMerger {
         this.read2BasesTrimmed = read2BasesTrimmed;
         this.expectedOrientations = expectedOrientations;
 
+        this.primaryAlignmentSelectionStrategy = primaryAlignmentSelectionStrategy;
     }
 
+    /** Allows the caller to override the maximum records in RAM. */
+    public void setMaxRecordsInRam(final int maxRecordsInRam) {
+        this.maxRecordsInRam = maxRecordsInRam;
+    }
+
+    /**
+     * Do this unconditionally, not just for aligned records, for two reasons:
+     * - An unaligned read has been processed by the aligner, so it is more truthful.
+     * - When chaining additional PG records, having all the records in the output file refer to the same PG
+     *   record means that only one chain will need to be created, rather than a chain for the mapped reads
+     *   and a separate chain for the unmapped reads.
+     */
+    private void maybeSetPgTag(final SAMRecord rec) {
+        if (this.programRecord != null) {
+            rec.setAttribute(ReservedTagConstants.PROGRAM_GROUP_ID, this.programRecord.getProgramGroupId());
+        }
+    }
+    /**
+
     /**
      * Merges the alignment data with the non-aligned records from the source BAM file.
      */
     public void mergeAlignment() {
-
         // Open the file of unmapped records and write the read groups to the the header for the merged file
         final SAMFileReader unmappedSam = new SAMFileReader(this.unmappedBamFile);
         final CloseableIterator<SAMRecord> unmappedIterator = unmappedSam.iterator();
@@ -164,49 +209,47 @@ public abstract class AbstractAlignmentMerger {
         int unmapped = 0;
 
         // Get the aligned records and set up the first one
-        alignedIterator = new MultiHitAlignedReadIterator(getQuerynameSortedAlignedRecords());
-        MultiHitAlignedReadIterator.HitsForInsert nextAligned = nextAligned();
+        alignedIterator = new MultiHitAlignedReadIterator(
+                new FilteringIterator(getQuerynameSortedAlignedRecords(), alignmentFilter),
+                primaryAlignmentSelectionStrategy);
+        HitsForInsert nextAligned = nextAligned();
 
-        // Create the sorting collection that will write the records in coordinate order
+        // Create the sorting collection that will write the records in the coordinate order
         // to the final bam file
-        final SortingCollection<SAMRecord> coordinateSorted = SortingCollection.newInstance(
+        final SortingCollection<SAMRecord> sorted = SortingCollection.newInstance(
             SAMRecord.class, new BAMRecordCodec(header), new SAMRecordCoordinateComparator(),
             MAX_RECORDS_IN_RAM);
 
         while (unmappedIterator.hasNext()) {
             // Load next unaligned read or read pair.
             final SAMRecord rec = unmappedIterator.next();
+
             rec.setHeader(this.header);
+            maybeSetPgTag(rec);
 
             final SAMRecord secondOfPair;
             if (rec.getReadPairedFlag()) {
                 secondOfPair = unmappedIterator.next();
                 secondOfPair.setHeader(this.header);
+                maybeSetPgTag(secondOfPair);
 
                 // Validate that paired reads arrive as first of pair followed by second of pair
-                if (!rec.getReadName().equals(secondOfPair.getReadName())) {
-                    throw new PicardException("Second read from pair not found in unmapped bam: " +
-                        rec.getReadName() + ", " + secondOfPair.getReadName());
-                }
-                if (!rec.getFirstOfPairFlag()) {
-                    throw new PicardException("First record in unmapped bam is not first of pair: " + rec.getReadName());
-                }
-                if (!secondOfPair.getReadPairedFlag()) {
-                    throw new PicardException("Second record in unmapped bam is not marked as paired: " + secondOfPair.getReadName());
-                }
-                if (!secondOfPair.getSecondOfPairFlag()) {
-                    throw new PicardException("Second record in unmapped bam is not second of pair: " + secondOfPair.getReadName());
-                }
-            } else {
+                if (!rec.getReadName().equals(secondOfPair.getReadName()))
+                    throw new PicardException("Second read from pair not found in unmapped bam: " + rec.getReadName() + ", " + secondOfPair.getReadName());
+
+                if (!rec.getFirstOfPairFlag()) throw new PicardException("First record in unmapped bam is not first of pair: " + rec.getReadName());
+                if (!secondOfPair.getReadPairedFlag())  throw new PicardException("Second record in unmapped bam is not marked as paired: " + secondOfPair.getReadName());
+                if (!secondOfPair.getSecondOfPairFlag())  throw new PicardException("Second record in unmapped bam is not second of pair: " + secondOfPair.getReadName());
+            }
+            else {
                 secondOfPair = null;
             }
 
             // See if there are alignments for current unaligned read or read pair.
             if (nextAligned != null && rec.getReadName().equals(nextAligned.getReadName())) {
-
                 // If there are multiple alignments for a read (pair), then the unaligned SAMRecord must be cloned
                 // before copying info from the aligned record to the unaligned.
-                boolean clone = nextAligned.numHits() > 1;
+                final boolean clone = nextAligned.numHits() > 1;
 
                 if (rec.getReadPairedFlag()) {
                     for (int i = 0; i < nextAligned.numHits(); ++i) {
@@ -221,36 +264,31 @@ public abstract class AbstractAlignmentMerger {
                         }
                         // firstAligned or secondAligned may be null, if there wasn't an alignment for the end,
                         // or if the alignment was rejected by ignoreAlignment.
-                        SAMRecord firstAligned = nextAligned.getFirstOfPair(i);
-                        SAMRecord secondAligned = nextAligned.getSecondOfPair(i);
+                        final SAMRecord firstAligned = nextAligned.getFirstOfPair(i);
+                        final SAMRecord secondAligned = nextAligned.getSecondOfPair(i);
 
-                        boolean isPrimaryAlignment = (firstAligned != null && !firstAligned.getNotPrimaryAlignmentFlag()) ||
+                        final boolean isPrimaryAlignment = (firstAligned != null && !firstAligned.getNotPrimaryAlignmentFlag()) ||
                                 (secondAligned != null && !secondAligned.getNotPrimaryAlignmentFlag());
 
                         transferAlignmentInfoToPairedRead(firstToWrite, secondToWrite, firstAligned, secondAligned);
 
                         // Only write unmapped read when it has the mate info from the primary alignment.
                         if (!firstToWrite.getReadUnmappedFlag() || isPrimaryAlignment) {
-                            coordinateSorted.add(firstToWrite);
+                            addIfNotFiltered(sorted, firstToWrite);
                             if (firstToWrite.getReadUnmappedFlag()) ++unmapped;
                             else ++aligned;
                         }
                         if (!secondToWrite.getReadUnmappedFlag() || isPrimaryAlignment) {
-                            coordinateSorted.add(secondToWrite);
+                            addIfNotFiltered(sorted, secondToWrite);
                             if (!secondToWrite.getReadUnmappedFlag()) ++aligned;
                             else ++unmapped;
                         }
                     }
                 } else {
                     for (int i = 0; i < nextAligned.numHits(); ++i) {
-                        final SAMRecord recToWrite;
-                        if (clone) {
-                            recToWrite = clone(rec);
-                        } else {
-                            recToWrite = rec;
-                        }
+                        final SAMRecord recToWrite = clone ? clone(rec) : rec;
                         transferAlignmentInfoToFragment(recToWrite, nextAligned.getFragment(i));
-                        coordinateSorted.add(recToWrite);
+                        addIfNotFiltered(sorted, recToWrite);
                         if (recToWrite.getReadUnmappedFlag()) ++unmapped;
                         else ++aligned;
                     }
@@ -265,14 +303,18 @@ public abstract class AbstractAlignmentMerger {
                 }
                 // No matching read from alignedIterator -- just output reads as is.
                 if (!alignedReadsOnly) {
-                    coordinateSorted.add(rec);
+                    sorted.add(rec);
                     ++unmapped;
                     if (secondOfPair != null) {
-                        coordinateSorted.add(secondOfPair);
+                        sorted.add(secondOfPair);
                         ++unmapped;
                     }
                 }
             }
+
+            if ((aligned + unmapped) % 1000000 == 0) {
+                log.info("Processed " + FMT.format(aligned + unmapped) + " records in query name order.");
+            }
         }
         unmappedIterator.close();
         if (alignedIterator.hasNext()) {
@@ -281,13 +323,15 @@ public abstract class AbstractAlignmentMerger {
         }
         alignedIterator.close();
 
-        // Write the records to the output file in coordinate sorted order,
-        final SAMFileWriter writer = new SAMFileWriterFactory().makeSAMOrBAMWriter(header, true, this.targetBamFile);
+        // Write the records to the output file in specified sorted order,
+        header.setSortOrder(this.sortOrder);
+        final boolean presorted = this.sortOrder == SortOrder.coordinate;
+        final SAMFileWriter writer = new SAMFileWriterFactory().makeSAMOrBAMWriter(header, presorted, this.targetBamFile);
         int count = 0;
-        for (final SAMRecord rec : coordinateSorted) {
+        for (final SAMRecord rec : sorted) {
             if (!rec.getReadUnmappedFlag()) {
                 if (refSeq != null) {
-                    byte referenceBases[] = refSeq.get(sequenceDictionary.getSequenceIndex(rec.getReferenceName())).getBases();
+                    final byte[] referenceBases = refSeq.get(sequenceDictionary.getSequenceIndex(rec.getReferenceName())).getBases();
                     rec.setAttribute(SAMTag.NM.name(),
                         SequenceUtil.calculateSamNmTag(rec, referenceBases, 0, bisulfiteSequence));
                     if (rec.getBaseQualities() != SAMRecord.NULL_QUALS) {
@@ -298,15 +342,21 @@ public abstract class AbstractAlignmentMerger {
             }
             writer.addAlignment(rec);
             if (++count % 1000000 == 0) {
-                log.info(count + " SAMRecords written to " + targetBamFile.getName());
+                log.info(FMT.format(count) + " SAMRecords written to " + targetBamFile.getName());
             }
         }
         writer.close();
-        coordinateSorted.cleanup();
+        sorted.cleanup();
 
         log.info("Wrote " + aligned + " alignment records and " + (alignedReadsOnly ? 0 : unmapped) + " unmapped reads.");
     }
 
+    private void addIfNotFiltered(final SortingCollection<SAMRecord> sorted, final SAMRecord rec) {
+        if (includeSecondaryAlignments || !rec.getNotPrimaryAlignmentFlag()) {
+            sorted.add(rec);
+        }
+    }
+
     private SAMRecord clone(final SAMRecord rec) {
         try {
             return (SAMRecord)rec.clone();
@@ -319,14 +369,8 @@ public abstract class AbstractAlignmentMerger {
      * The alignments are run through ignoreAlignment() filter before being returned, which may result
      * in an entire read being skipped if all alignments for that read should be ignored.
      */
-    private MultiHitAlignedReadIterator.HitsForInsert nextAligned() {
-        while (alignedIterator.hasNext()) {
-            MultiHitAlignedReadIterator.HitsForInsert hits = alignedIterator.next();
-            hits.filterReads(alignmentFilter);
-            if (hits.numHits() > 0) {
-                return hits;
-            }
-        }
+    private HitsForInsert nextAligned() {
+        if (alignedIterator.hasNext()) return alignedIterator.next();
         return null;
     }
 
@@ -335,12 +379,11 @@ public abstract class AbstractAlignmentMerger {
      * @param unaligned Original SAMRecord, and object into which values are copied.
      * @param aligned Holds alignment info that will be copied into unaligned.
      */
-    private void transferAlignmentInfoToFragment(SAMRecord unaligned, SAMRecord aligned) {
+    private void transferAlignmentInfoToFragment(final SAMRecord unaligned, final SAMRecord aligned) {
         setValuesFromAlignment(unaligned, aligned);
         updateCigarForTrimmedOrClippedBases(unaligned, aligned);
-        if (this.programRecord != null) {
-            unaligned.setAttribute(ReservedTagConstants.PROGRAM_GROUP_ID,
-                this.programRecord.getProgramGroupId());
+        if (SAMUtils.cigarMapsNoBasesToRef(unaligned.getCigar())) {
+            SAMUtils.makeReadUnmapped(unaligned);
         }
     }
 
@@ -351,11 +394,14 @@ public abstract class AbstractAlignmentMerger {
      * @param firstAligned Aligned first of pair, or null if no alignment.
      * @param secondAligned Aligned second of pair, or null if no alignment.
      */
-    private void transferAlignmentInfoToPairedRead(SAMRecord firstUnaligned, SAMRecord secondUnaligned, SAMRecord firstAligned, SAMRecord secondAligned) {
+    private void transferAlignmentInfoToPairedRead(final SAMRecord firstUnaligned, final SAMRecord secondUnaligned, final SAMRecord firstAligned, final SAMRecord secondAligned) {
         if (firstAligned != null) transferAlignmentInfoToFragment(firstUnaligned, firstAligned);
         if (secondAligned != null) transferAlignmentInfoToFragment(secondUnaligned, secondAligned);
-        clipForOverlappingReads(firstUnaligned, secondUnaligned);
-        SamPairUtil.setProperPairAndMateInfo(secondUnaligned, firstUnaligned, header, expectedOrientations);
+        if (isClipOverlappingReads()) clipForOverlappingReads(firstUnaligned, secondUnaligned);
+        SamPairUtil.setMateInfo(secondUnaligned, firstUnaligned, header);
+        if (!keepAlignerProperPairFlags) {
+            SamPairUtil.setProperPairFlags(secondUnaligned, firstUnaligned, expectedOrientations);
+        }
     }
 
 
@@ -444,21 +490,21 @@ public abstract class AbstractAlignmentMerger {
     protected void updateCigarForTrimmedOrClippedBases(final SAMRecord rec, final SAMRecord alignment) {
 
         // If the read maps off the end of the alignment, clip it
-        SAMSequenceRecord refseq = rec.getHeader().getSequence(rec.getReferenceIndex());
+        final SAMSequenceRecord refseq = rec.getHeader().getSequence(rec.getReferenceIndex());
         if (rec.getAlignmentEnd() > refseq.getSequenceLength()) {
             // 1-based index of first base in read to clip.
-            int clipFrom = refseq.getSequenceLength() - rec.getAlignmentStart() + 1;
-            List<CigarElement> newCigarElements  = CigarUtil.softClipEndOfRead(clipFrom, rec.getCigar().getCigarElements());
+            final int clipFrom = refseq.getSequenceLength() - rec.getAlignmentStart() + 1;
+            final List<CigarElement> newCigarElements  = CigarUtil.softClipEndOfRead(clipFrom, rec.getCigar().getCigarElements());
             rec.setCigar(new Cigar(newCigarElements));
         }
 
         // If the read was trimmed or not all the bases were sent for alignment, clip it
-        int alignmentReadLength = alignment.getReadLength();
-        int originalReadLength = rec.getReadLength();
-        int trimmed = (!rec.getReadPairedFlag()) || rec.getFirstOfPairFlag()
+        final int alignmentReadLength = alignment.getReadLength();
+        final int originalReadLength = rec.getReadLength();
+        final int trimmed = (!rec.getReadPairedFlag()) || rec.getFirstOfPairFlag()
                 ? this.read1BasesTrimmed != null ? this.read1BasesTrimmed : 0
                 : this.read2BasesTrimmed != null ? this.read2BasesTrimmed : 0;
-        int notWritten = originalReadLength - (alignmentReadLength + trimmed);
+        final int notWritten = originalReadLength - (alignmentReadLength + trimmed);
 
         rec.setCigar(CigarUtil.addSoftClippedBasesToEndsOfCigar(
             rec.getCigar(), rec.getReadNegativeStrandFlag(), notWritten, trimmed));
@@ -474,7 +520,7 @@ public abstract class AbstractAlignmentMerger {
 
     protected SAMProgramRecord getProgramRecord() { return this.programRecord; }
 
-    protected void setProgramRecord(SAMProgramRecord pg ) {
+    protected void setProgramRecord(final SAMProgramRecord pg ) {
         if (this.programRecord != null) {
             throw new IllegalStateException("Cannot set program record more than once on alignment merger.");
         }
@@ -483,14 +529,14 @@ public abstract class AbstractAlignmentMerger {
         SAMUtils.chainSAMProgramRecord(header, pg);
     }
 
-    protected boolean isReservedTag(String tag) {
+    protected boolean isReservedTag(final String tag) {
         final char firstCharOfTag = tag.charAt(0);
 
         // All tags that start with a lower-case letter are user defined and should not be overridden by aligner
         // unless explicitly specified in attributesToRetain.
         if (Character.isLowerCase(firstCharOfTag)) return true;
 
-        for (char c : RESERVED_ATTRIBUTE_STARTS) {
+        for (final char c : RESERVED_ATTRIBUTE_STARTS) {
             if (firstCharOfTag == c) return true;
         }
         return false;
@@ -501,4 +547,27 @@ public abstract class AbstractAlignmentMerger {
     protected void resetRefSeqFileWalker() {
         this.refSeq = new ReferenceSequenceFileWalker(referenceFasta);
     }
+
+    public boolean isClipOverlappingReads() {
+        return clipOverlappingReads;
+    }
+
+    public void setClipOverlappingReads(final boolean clipOverlappingReads) {
+        this.clipOverlappingReads = clipOverlappingReads;
+    }
+
+    public boolean isKeepAlignerProperPairFlags() {
+        return keepAlignerProperPairFlags;
+    }
+
+    /**
+     * If true, keep the aligner's idea of proper pairs rather than letting alignment merger decide.
+     */
+    public void setKeepAlignerProperPairFlags(final boolean keepAlignerProperPairFlags) {
+        this.keepAlignerProperPairFlags = keepAlignerProperPairFlags;
+    }
+
+    public void setIncludeSecondaryAlignments(final boolean includeSecondaryAlignments) {
+        this.includeSecondaryAlignments = includeSecondaryAlignments;
+    }
 }
