@@ -31,15 +31,15 @@ import net.sf.picard.cmdline.Usage;
 import net.sf.picard.fastq.FastqReader;
 import net.sf.picard.fastq.FastqRecord;
 import net.sf.picard.io.IoUtil;
-import net.sf.picard.util.FastqQualityFormat;
-import net.sf.picard.util.Log;
-import net.sf.picard.util.SolexaQualityConverter;
+import net.sf.picard.util.*;
 import net.sf.samtools.*;
 import net.sf.samtools.SAMFileHeader.SortOrder;
 import net.sf.samtools.util.Iso8601Date;
 import net.sf.samtools.util.StringUtil;
 
 import java.io.File;
+import java.util.ArrayList;
+import java.util.List;
 
 /**
  * Converts a fastq file to an unaligned BAM/SAM format.
@@ -63,7 +63,7 @@ public class FastqToSam extends CommandLineProgram {
 
     @Option(shortName="V", doc="A value describing how the quality values are encoded in the fastq.  Either Solexa for pre-pipeline 1.3 " +
             "style scores (solexa scaling + 66), Illumina for pipeline 1.3 and above (phred scaling + 64) or Standard for phred scaled " +
-            "scores with a character shift of 33.")
+            "scores with a character shift of 33.  If this value is not specified, the quality format will be detected automatically.", optional = true)
     public FastqQualityFormat QUALITY_FORMAT;
 
     @Option(doc="Output SAM/BAM file. ", shortName=StandardOptionDefinitions.OUTPUT_SHORT_NAME) 
@@ -90,6 +90,9 @@ public class FastqToSam extends CommandLineProgram {
     @Option(shortName = "PI", doc = "Predicted median insert size, to insert into the read group header", optional = true)
     public Integer PREDICTED_INSERT_SIZE;
 
+    @Option(doc="Comment(s) to include in the merged output file's header.", optional=true, shortName="CO")
+    public List<String> COMMENT = new ArrayList<String>();
+
     @Option(shortName = "DS", doc = "Inserted into the read group header", optional = true)
     public String DESCRIPTION;
 
@@ -99,6 +102,15 @@ public class FastqToSam extends CommandLineProgram {
     @Option(shortName="SO", doc="The sort order for the output sam/bam file.")
     public SortOrder SORT_ORDER = SortOrder.queryname;
 
+    @Option(doc="Minimum quality allowed in the input fastq.  An exception will be thrown if a quality is less than this value.")
+    public int MIN_Q = 0;
+
+    @Option(doc="Maximum quality allowed in the input fastq.  An exception will be thrown if a quality is greater than this value.")
+    public int MAX_Q = SAMUtils.MAX_PHRED_SCORE;
+
+    @Option(doc="If true and this is an unpaired fastq any occurance of '/1' will be removed from the end of a read name.")
+    public Boolean STRIP_UNPAIRED_MATE_NUMBER = false;
+
     private static final SolexaQualityConverter solexaQualityConverter = SolexaQualityConverter.getSingleton();
 
     /** Stock main method. */
@@ -108,6 +120,23 @@ public class FastqToSam extends CommandLineProgram {
 
     /* Simply invokes the right method for unpaired or paired data. */
     protected int doWork() {
+        if (QUALITY_FORMAT == null) {
+            final QualityEncodingDetector detector = new QualityEncodingDetector();
+            final FastqReader reader = new FastqReader(FASTQ);
+            if (FASTQ2 == null) {
+                detector.add(QualityEncodingDetector.DEFAULT_MAX_RECORDS_TO_ITERATE, reader);
+            } else {
+                final FastqReader reader2 = new FastqReader(FASTQ2);       
+                detector.add(QualityEncodingDetector.DEFAULT_MAX_RECORDS_TO_ITERATE, reader, reader2);
+                reader2.close();
+            }
+            reader.close();
+            
+            QUALITY_FORMAT = detector.generateBestGuess(QualityEncodingDetector.FileContext.FASTQ);
+            if (detector.isDeterminationAmbiguous())
+                LOG.warn("Making ambiguous determination about fastq's quality encoding; more than one format possible based on observed qualities.");
+            LOG.info(String.format("Auto-detected quality format as: %s.", QUALITY_FORMAT));
+        }
         final int readCount = (FASTQ2 == null) ?  doUnpaired() : doPaired();
         LOG.info("Processed " + readCount + " fastq reads");
         return 0;
@@ -123,11 +152,13 @@ public class FastqToSam extends CommandLineProgram {
         final SAMFileWriter writer = new SAMFileWriterFactory().makeSAMOrBAMWriter(header, false, OUTPUT);
 
         int readCount = 0;
+        final ProgressLogger progress = new ProgressLogger(LOG);
         for ( ; freader.hasNext()  ; readCount++) {
             final FastqRecord frec = freader.next();
-            final SAMRecord srec = createSamRecord(header, getReadName(frec.getReadHeader()) , frec, false) ;
+            final SAMRecord srec = createSamRecord(header, getReadName(frec.getReadHeader(), false) , frec, false) ;
             srec.setReadPairedFlag(false);
             writer.addAlignment(srec);
+            progress.record(srec);
         }
 
         writer.close();
@@ -146,23 +177,26 @@ public class FastqToSam extends CommandLineProgram {
         final SAMFileWriter writer = (new SAMFileWriterFactory()).makeSAMOrBAMWriter(header, false, OUTPUT);
 
         int readCount = 0;
+        final ProgressLogger progress = new ProgressLogger(LOG);
         for ( ; freader1.hasNext() && freader2.hasNext() ; readCount++) {
             final FastqRecord frec1 = freader1.next();
             final FastqRecord frec2 = freader2.next();
 
-            final String frec1Name = getReadName(frec1.getReadHeader());
-            final String frec2Name = getReadName(frec2.getReadHeader());
+            final String frec1Name = getReadName(frec1.getReadHeader(), true);
+            final String frec2Name = getReadName(frec2.getReadHeader(), true);
             final String baseName = getBaseName(frec1Name, frec2Name, freader1, freader2);
 
             final SAMRecord srec1 = createSamRecord(header, baseName, frec1, true) ;
             srec1.setFirstOfPairFlag(true);
             srec1.setSecondOfPairFlag(false);
             writer.addAlignment(srec1);
+            progress.record(srec1);
 
             final SAMRecord srec2 = createSamRecord(header, baseName, frec2, true) ;
             srec2.setFirstOfPairFlag(false);
             srec2.setSecondOfPairFlag(true);
             writer.addAlignment(srec2);
+            progress.record(srec2);
         }
 
         writer.close();
@@ -182,6 +216,13 @@ public class FastqToSam extends CommandLineProgram {
         srec.setAttribute(ReservedTagConstants.READ_GROUP_ID, READ_GROUP_NAME);
         final byte[] quals = StringUtil.stringToBytes(frec.getBaseQualityString());
         convertQuality(quals, QUALITY_FORMAT);
+        for (final byte qual : quals) {
+            final int uQual = qual & 0xff;
+            if (uQual < MIN_Q || uQual > MAX_Q) {
+                throw new PicardException("Base quality " + uQual + " is not in the range " + MIN_Q + ".." +
+                MAX_Q + " for read " + frec.getReadHeader());
+            }
+        }
         srec.setBaseQualities(quals);
 
         if (paired) {
@@ -205,6 +246,11 @@ public class FastqToSam extends CommandLineProgram {
 
         final SAMFileHeader header = new SAMFileHeader();
         header.addReadGroup(rgroup);
+
+        for (String comment : COMMENT) {
+            header.addComment(comment);
+        }
+
         header.setSortOrder(this.SORT_ORDER);
         return header ;
     }
@@ -289,8 +335,25 @@ public class FastqToSam extends CommandLineProgram {
     }
 
     // Read names cannot contain blanks
-    private String getReadName(final String fastaqHeader) {
-        final int idx = fastaqHeader.indexOf(" ");
-        return (idx == -1) ? fastaqHeader : fastaqHeader.substring(0,idx); 
+    private String getReadName(final String fastqHeader, final boolean paired) {
+        final int idx = fastqHeader.indexOf(" ");
+        String readName = (idx == -1) ? fastqHeader : fastqHeader.substring(0,idx);
+
+        // NOTE: the while loop isn't necessarily the most efficient way to handle this but we don't
+        // expect this to ever happen more than once, just trapping pathological cases
+        while (STRIP_UNPAIRED_MATE_NUMBER && !paired && readName.endsWith("/1")) {
+            // If this is an unpaired run we want to make sure that "/1" isn't tacked on the end of the read name,
+            // as this can cause problems down the road in MergeBamAlignment
+            readName = readName.substring(0, readName.length() - 2);
+        }
+
+        return readName;
+    }
+
+    @Override
+    protected String[] customCommandLineValidation() {
+        if (MIN_Q < 0) return new String[]{"MIN_Q must be >= 0"};
+        if (MAX_Q > SAMUtils.MAX_PHRED_SCORE) return new String[]{"MAX_Q must be <= " + SAMUtils.MAX_PHRED_SCORE};
+        return null;
     }
 }
