@@ -34,6 +34,7 @@ import java.util.Iterator;
 import java.util.List;
 
 import net.sf.picard.util.PeekableIterator;
+import net.sf.picard.util.ProgressLogger;
 import net.sf.samtools.*;
 import net.sf.samtools.SAMFileHeader.SortOrder;
 import net.sf.picard.util.Log;
@@ -53,7 +54,8 @@ import net.sf.samtools.util.SortingCollection;
 public class FixMateInformation extends CommandLineProgram {
     @Usage public final String USAGE = "Ensure that all mate-pair information is in sync between each read " +
             " and it's mate pair.  If no OUTPUT file is supplied then the output is written to a temporary file " +
-            " and then copied over the INPUT file.";
+            " and then copied over the INPUT file.  Reads marked with the secondary alignment flag are written " +
+            "to the output file unchanged.";
 
     @Option(shortName=StandardOptionDefinitions.INPUT_SHORT_NAME, doc="The input file to fix.")
     public List<File> INPUT;
@@ -80,8 +82,8 @@ public class FixMateInformation extends CommandLineProgram {
         final List<SAMFileReader> readers = new ArrayList<SAMFileReader>();
         for (final File f : INPUT) {
             IoUtil.assertFileIsReadable(f);
-            SAMFileReader reader = new SAMFileReader(f);
-            readers.add(new SAMFileReader(f));
+            final SAMFileReader reader = new SAMFileReader(f);
+            readers.add(reader);
             if (reader.getFileHeader().getSortOrder() != SortOrder.queryname) allQueryNameSorted = false;
         }
 
@@ -115,10 +117,10 @@ public class FixMateInformation extends CommandLineProgram {
 
         {
             // Deal with merging if necessary
-            Iterator<SAMRecord> tmp;
+            final Iterator<SAMRecord> tmp;
             if (INPUT.size() > 1) {
-                List<SAMFileHeader> headers = new ArrayList<SAMFileHeader>(readers.size());
-                for (SAMFileReader reader : readers) {
+                final List<SAMFileHeader> headers = new ArrayList<SAMFileHeader>(readers.size());
+                for (final SAMFileReader reader : readers) {
                     headers.add(reader.getFileHeader());
                 }
                 final SortOrder sortOrder = (allQueryNameSorted? SortOrder.queryname: SortOrder.unsorted);
@@ -170,25 +172,39 @@ public class FixMateInformation extends CommandLineProgram {
         createSamFileWriter(header);
 
         log.info("Traversing query name sorted records and fixing up mate pair information.");
-        long count = 0;
+        final ProgressLogger progress = new ProgressLogger(log);
         while (iterator.hasNext()) {
             final SAMRecord rec1 = iterator.next();
-            final SAMRecord rec2 = iterator.hasNext() ? iterator.peek() : null;
+            if (rec1.getNotPrimaryAlignmentFlag()) {
+                writeAlignment(rec1);
+                progress.record(rec1);
+                continue;
+            }
+            SAMRecord rec2 = null;
+            // Keep peeking at next SAMRecord until one is found that is not marked as secondary alignment,
+            // or until there are no more SAMRecords.
+            while (iterator.hasNext()) {
+                rec2 = iterator.peek();
+                if (rec2.getNotPrimaryAlignmentFlag()) {
+                    iterator.next();
+                    writeAlignment(rec2);
+                    progress.record(rec2);
+                    rec2 = null;
+                } else {
+                    break;
+                }
+            }
 
             if (rec2 != null && rec1.getReadName().equals(rec2.getReadName())) {
                 iterator.next(); // consume the peeked record
                 SamPairUtil.setMateInfo(rec1, rec2, header);
                 writeAlignment(rec1);
                 writeAlignment(rec2);
-                count += 2;
+                progress.record(rec1, rec2);
             }
             else {
                 writeAlignment(rec1);
-                ++count;
-            }
-
-            if (count % 1000000 == 0) {
-                log.info("Processed " + count + " records.");
+                progress.record(rec1);
             }
         }
         iterator.close();
@@ -249,13 +265,13 @@ public class FixMateInformation extends CommandLineProgram {
         return 0;
     }
 
-    protected void createSamFileWriter(SAMFileHeader header) {
+    protected void createSamFileWriter(final SAMFileHeader header) {
         out = new SAMFileWriterFactory().makeSAMOrBAMWriter(header,
                   header.getSortOrder() == SortOrder.queryname, OUTPUT);
 
     }
 
-    protected void writeAlignment(SAMRecord sam) {
+    protected void writeAlignment(final SAMRecord sam) {
         out.addAlignment(sam);
     }
 
