@@ -24,6 +24,7 @@
 
 package net.sf.picard.sam;
 
+import net.sf.picard.cmdline.CommandLineParser;
 import net.sf.picard.cmdline.Option;
 import net.sf.picard.cmdline.StandardOptionDefinitions;
 import net.sf.picard.cmdline.Usage;
@@ -32,7 +33,10 @@ import net.sf.picard.util.Histogram;
 import net.sf.picard.util.Log;
 import net.sf.picard.PicardException;
 import net.sf.picard.io.IoUtil;
+import net.sf.picard.util.ProgressLogger;
 import net.sf.samtools.*;
+import net.sf.samtools.SAMFileHeader.SortOrder;
+import net.sf.samtools.util.CloseableIterator;
 import net.sf.samtools.util.SortingCollection;
 import net.sf.samtools.util.SortingLongCollection;
 
@@ -53,34 +57,71 @@ public class MarkDuplicates extends AbstractDuplicateFindingAlgorithm {
      * be enough file handles.
      */
 
-    @Usage public final String USAGE =
+    @Usage
+    public final String USAGE =
+		    CommandLineParser.getStandardUsagePreamble(getClass()) +
             "Examines aligned records in the supplied SAM or BAM file to locate duplicate molecules. " +
             "All records are then written to the output file with the duplicate records flagged.";
 
-    @Option(shortName= StandardOptionDefinitions.INPUT_SHORT_NAME, doc="The input SAM or BAM file to analyze.  Must be coordinate sorted.")
-    public File INPUT;
+    @Option(shortName=StandardOptionDefinitions.INPUT_SHORT_NAME,
+		    doc="One or more input SAM or BAM files to analyze. Must be coordinate sorted.")
+    public List<File> INPUT;
 
-    @Option(shortName=StandardOptionDefinitions.OUTPUT_SHORT_NAME, doc="The output file to right marked records to")
+    @Option(shortName=StandardOptionDefinitions.OUTPUT_SHORT_NAME,
+		    doc="The output file to right marked records to")
     public File OUTPUT;
 
-    @Option(shortName="M", doc="File to write duplication metrics to")
+    @Option(shortName="M",
+		    doc="File to write duplication metrics to")
     public File METRICS_FILE;
 
+    @Option(shortName=StandardOptionDefinitions.PROGRAM_RECORD_ID_SHORT_NAME,
+            doc="The program record ID for the @PG record(s) created by this program. Set to null to disable " +
+                "PG record creation.  This string may have a suffix appended to avoid collision with other " +
+	            "program record IDs.",
+            optional=true)
+    public String PROGRAM_RECORD_ID = "MarkDuplicates";
+
+    @Option(shortName="PG_VERSION",
+            doc="Value of VN tag of PG record to be created. If not specified, the version will be detected automatically.",
+            optional=true)
+    public String PROGRAM_GROUP_VERSION;
+
+    @Option(shortName="PG_COMMAND",
+            doc="Value of CL tag of PG record to be created. If not supplied the command line will be detected automatically.",
+            optional=true)
+    public String PROGRAM_GROUP_COMMAND_LINE;
+
+    @Option(shortName="PG_NAME",
+		    doc="Value of PN tag of PG record to be created.")
+    public String PROGRAM_GROUP_NAME = "MarkDuplicates";
+
+    @Option(shortName="CO",
+		    doc="Comment(s) to include in the output file's header.",
+		    optional=true)
+    public List<String> COMMENT = new ArrayList<String>();
+
     @Option(doc="If true do not write duplicates to the output file instead of writing them with appropriate flags set.")
     public boolean REMOVE_DUPLICATES = false;
 
-    @Option(doc="If true, assume that the input file is coordinate sorted, even if the header says otherwise.",
-            shortName=StandardOptionDefinitions.ASSUME_SORTED_SHORT_NAME)
+    @Option(shortName=StandardOptionDefinitions.ASSUME_SORTED_SHORT_NAME,
+		    doc="If true, assume that the input file is coordinate sorted even if the header says otherwise.")
     public boolean ASSUME_SORTED = false;
 
-    @Option(doc="This option is obsolete.  ReadEnds will always be spilled to disk.", shortName="MAX_SEQS")
+    @Option(shortName="MAX_SEQS",
+		    doc="This option is obsolete. ReadEnds will always be spilled to disk.")
     public int MAX_SEQUENCES_FOR_DISK_READ_ENDS_MAP = 50000;
 
-    @Option(doc="Maximum number of file handles to keep open when spilling read ends to disk.  " + "" +
-            "Set this number a little lower than the per-process maximum number of file that may be open.  " +
-            "This number can be found by executing the 'ulimit -n' command on a Unix system.", shortName = "MAX_FILE_HANDLES")
+    @Option(shortName="MAX_FILE_HANDLES",
+		    doc="Maximum number of file handles to keep open when spilling read ends to disk. " +
+            "Set this number a little lower than the per-process maximum number of file that may be open. " +
+            "This number can be found by executing the 'ulimit -n' command on a Unix system.")
     public int MAX_FILE_HANDLES_FOR_READ_ENDS_MAP = 8000;
 
+    @Option(doc="This number, plus the maximum RAM available to the JVM, determine the memory footprint used by " +
+            "some of the sorting collections.  If you are running out of memory, try reducing this number.")
+    public double SORTING_COLLECTION_SIZE_RATIO = 0.25;
+
     private SortingCollection<ReadEnds> pairSort;
     private SortingCollection<ReadEnds> fragSort;
     private SortingLongCollection duplicateIndexes;
@@ -92,6 +133,13 @@ public class MarkDuplicates extends AbstractDuplicateFindingAlgorithm {
     // Variables used for optical duplicate detection and tracking
     private final Histogram<Short> opticalDupesByLibraryId = new Histogram<Short>();
 
+    // All PG IDs seen in merged input files in first pass.  These are gather for two reasons:
+    // - to know how many different PG records to create to represent this program invocation.
+    // - to know what PG IDs are already used to avoid collisions when creating new ones.
+    // Note that if there are one or more records that do not have a PG tag, then a null value
+    // will be stored in this set.
+    private final Set<String> pgIdsSeen = new HashSet<String>();
+
     /** Stock main method. */
     public static void main(final String[] args) {
         System.exit(new MarkDuplicates().instanceMain(args));
@@ -104,7 +152,7 @@ public class MarkDuplicates extends AbstractDuplicateFindingAlgorithm {
      * input file and writing it out with duplication flags set correctly.
      */
     protected int doWork() {
-        IoUtil.assertFileIsReadable(INPUT);
+        for (final File f : INPUT) IoUtil.assertFileIsReadable(f);
         IoUtil.assertFileIsWritable(OUTPUT);
         IoUtil.assertFileIsWritable(METRICS_FILE);
 
@@ -118,31 +166,61 @@ public class MarkDuplicates extends AbstractDuplicateFindingAlgorithm {
         log.info("Found " + ((long) this.opticalDupesByLibraryId.getSumOfValues()) + " optical duplicate clusters.");
 
         final Map<String,DuplicationMetrics> metricsByLibrary = new HashMap<String,DuplicationMetrics>();
-        final SAMFileReader in  = new SAMFileReader(INPUT);
-        final SAMFileHeader header = in.getFileHeader();
+        final SamHeaderAndIterator headerAndIterator = openInputs();
+        final SAMFileHeader header = headerAndIterator.header;
+
         final SAMFileHeader outputHeader = header.clone();
         outputHeader.setSortOrder(SAMFileHeader.SortOrder.coordinate);
+        for (final String comment : COMMENT) outputHeader.addComment(comment);
+
+        // Key: previous PG ID on a SAM Record (or null).  Value: New PG ID to replace it.
+        final Map<String, String> chainedPgIds;
+        // Generate new PG record(s)
+        if (PROGRAM_RECORD_ID != null) {
+            final PgIdGenerator pgIdGenerator = new PgIdGenerator(outputHeader);
+            if (PROGRAM_GROUP_VERSION == null) {
+                PROGRAM_GROUP_VERSION = this.getVersion();
+            }
+            if (PROGRAM_GROUP_COMMAND_LINE == null) {
+                PROGRAM_GROUP_COMMAND_LINE = this.getCommandLine();
+            }
+            chainedPgIds = new HashMap<String, String>();
+            for (final String existingId : pgIdsSeen) {
+                final String newPgId = pgIdGenerator.getNonCollidingId(PROGRAM_RECORD_ID);
+                chainedPgIds.put(existingId, newPgId);
+                final SAMProgramRecord programRecord = new SAMProgramRecord(newPgId);
+                programRecord.setProgramVersion(PROGRAM_GROUP_VERSION);
+                programRecord.setCommandLine(PROGRAM_GROUP_COMMAND_LINE);
+                programRecord.setProgramName(PROGRAM_GROUP_NAME);
+                programRecord.setPreviousProgramGroupId(existingId);
+                outputHeader.addProgramRecord(programRecord);
+            }
+        } else {
+            chainedPgIds = null;
+        }
+
         final SAMFileWriter out = new SAMFileWriterFactory().makeSAMOrBAMWriter(outputHeader,
-                                                                          true,
-                                                                          OUTPUT);
+                                                                                true,
+                                                                                OUTPUT);
 
         // Now copy over the file while marking all the necessary indexes as duplicates
         long recordInFileIndex = 0;
         long nextDuplicateIndex = (this.duplicateIndexes.hasNext() ? this.duplicateIndexes.next(): -1);
 
-        if(header != null) {
-            for(final SAMReadGroupRecord readGroup : header.getReadGroups()) {
-                final String library = readGroup.getLibrary();
-                DuplicationMetrics metrics = metricsByLibrary.get(library);
-                if (metrics == null) {
-                    metrics = new DuplicationMetrics();
-                    metrics.LIBRARY = library;
-                    metricsByLibrary.put(library, metrics);
-                }
+        for(final SAMReadGroupRecord readGroup : header.getReadGroups()) {
+            final String library = readGroup.getLibrary();
+            DuplicationMetrics metrics = metricsByLibrary.get(library);
+            if (metrics == null) {
+                metrics = new DuplicationMetrics();
+                metrics.LIBRARY = library;
+                metricsByLibrary.put(library, metrics);
             }
         }
 
-        for (final SAMRecord rec : in) {
+        final ProgressLogger progress = new ProgressLogger(log, (int) 1e7, "Written");
+        final CloseableIterator<SAMRecord> iterator = headerAndIterator.iterator;
+        while (iterator.hasNext()) {
+            final SAMRecord rec = iterator.next();
             if (!rec.getNotPrimaryAlignmentFlag()) {
                 final String library = getLibraryName(header, rec);
                 DuplicationMetrics metrics = metricsByLibrary.get(library);
@@ -193,7 +271,11 @@ public class MarkDuplicates extends AbstractDuplicateFindingAlgorithm {
                 // do nothing
             }
             else {
+                if (PROGRAM_RECORD_ID != null) {
+                    rec.setAttribute(SAMTag.PG.name(), chainedPgIds.get(rec.getStringAttribute(SAMTag.PG.name())));
+                }
                 out.addAlignment(rec);
+                progress.record(rec);
             }
         }
 
@@ -214,9 +296,9 @@ public class MarkDuplicates extends AbstractDuplicateFindingAlgorithm {
             metrics.READ_PAIR_DUPLICATES = metrics.READ_PAIR_DUPLICATES / 2;
 
             // Add the optical dupes to the metrics
-            Short libraryId = this.libraryIds.get(libraryName);
+            final Short libraryId = this.libraryIds.get(libraryName);
             if (libraryId != null) {
-                Histogram<Short>.Bin bin = this.opticalDupesByLibraryId.get(libraryId);
+                final Histogram<Short>.Bin bin = this.opticalDupesByLibraryId.get(libraryId);
                 if (bin != null) {
                     metrics.READ_PAIR_OPTICAL_DUPLICATES = (long) bin.getValue();
                 }
@@ -234,6 +316,48 @@ public class MarkDuplicates extends AbstractDuplicateFindingAlgorithm {
         return 0;
     }
 
+    /** Little class used to package up a header and an iterable/iterator. */
+    private static final class SamHeaderAndIterator {
+        final SAMFileHeader header;
+        final CloseableIterator<SAMRecord> iterator;
+
+        private SamHeaderAndIterator(final SAMFileHeader header, final CloseableIterator<SAMRecord> iterator) {
+            this.header = header;
+            this.iterator = iterator;
+        }
+    }
+
+    /**
+     * Since MarkDuplicates reads it's inputs more than once this method does all the opening
+     * and checking of the inputs.
+     */
+    private SamHeaderAndIterator openInputs() {
+        final List<SAMFileHeader> headers = new ArrayList<SAMFileHeader>(INPUT.size());
+        final List<SAMFileReader> readers = new ArrayList<SAMFileReader>(INPUT.size());
+
+        for (final File f : INPUT) {
+            final SAMFileReader reader = new SAMFileReader(f);
+            final SAMFileHeader header = reader.getFileHeader();
+
+            if (!ASSUME_SORTED && header.getSortOrder() != SortOrder.coordinate) {
+                throw new PicardException("Input file " + f.getAbsolutePath() + " is not coordinate sorted.");
+            }
+
+            headers.add(header);
+            readers.add(reader);
+        }
+
+        if (headers.size() == 1) {
+            return new SamHeaderAndIterator(headers.get(0), readers.get(0).iterator());
+        }
+        else {
+            final SamFileHeaderMerger headerMerger = new SamFileHeaderMerger(SortOrder.coordinate, headers, false);
+            final MergingSamRecordIterator iterator = new MergingSamRecordIterator(headerMerger, readers, ASSUME_SORTED);
+            return new SamHeaderAndIterator(headerMerger.getMergedHeader(), iterator);
+        }
+    }
+
+    /** Print out some quick JVM memory stats. */
     private void reportMemoryStats(final String stage) {
         System.gc();
         final Runtime runtime = Runtime.getRuntime();
@@ -247,33 +371,37 @@ public class MarkDuplicates extends AbstractDuplicateFindingAlgorithm {
      * duplication, caching to disk as necssary to sort them.
      */
     private void buildSortedReadEndLists() {
-        final int maxInMemory = (int) ((Runtime.getRuntime().maxMemory() * 0.25) / ReadEnds.SIZE_OF);
+        final int maxInMemory = (int) ((Runtime.getRuntime().maxMemory() * SORTING_COLLECTION_SIZE_RATIO) / ReadEnds.SIZE_OF);
         log.info("Will retain up to " + maxInMemory + " data points before spilling to disk.");
 
         this.pairSort = SortingCollection.newInstance(ReadEnds.class,
                                                       new ReadEndsCodec(),
                                                       new ReadEndsComparator(),
-                                                      maxInMemory);
+                                                      maxInMemory,
+                                                      TMP_DIR);
 
         this.fragSort = SortingCollection.newInstance(ReadEnds.class,
                                                       new ReadEndsCodec(),
                                                       new ReadEndsComparator(),
-                                                      maxInMemory);
+                                                      maxInMemory,
+                                                      TMP_DIR);
 
-        final SAMFileReader sam = new SAMFileReader(INPUT);
-        final SAMFileHeader header = sam.getFileHeader();
-        if (header.getSortOrder() != SAMFileHeader.SortOrder.coordinate) {
-            if (ASSUME_SORTED) {
-                log.info("Assuming input is coordinate sorted.");
-            } else {
-                throw new PicardException(INPUT + " is not coordinate sorted.");
-            }
-        }
+        final SamHeaderAndIterator headerAndIterator = openInputs();
+        final SAMFileHeader header = headerAndIterator.header;
         final ReadEndsMap tmp = new DiskReadEndsMap(MAX_FILE_HANDLES_FOR_READ_ENDS_MAP);
-
         long index = 0;
+        final ProgressLogger progress = new ProgressLogger(log, (int) 1e6, "Read");
+        final CloseableIterator<SAMRecord> iterator = headerAndIterator.iterator;
+
+        while (iterator.hasNext()) {
+            final SAMRecord rec = iterator.next();
+
+            // This doesn't have anything to do with building sorted ReadEnd lists, but it can be done in the same pass
+            // over the input
+            if (PROGRAM_RECORD_ID != null) {
+                pgIdsSeen.add(rec.getStringAttribute(SAMTag.PG.name()));
+            }
 
-        for (final SAMRecord rec : sam) {
             if (rec.getReadUnmappedFlag()) {
                 if (rec.getReferenceIndex() == -1) {
                     // When we hit the unmapped reads with no coordinate, no reason to continue.
@@ -325,14 +453,14 @@ public class MarkDuplicates extends AbstractDuplicateFindingAlgorithm {
             }
 
             // Print out some stats every 1m reads
-            if (++index % 1000000 == 0) {
-                log.info("Read " + index + " records. Tracking " + tmp.size() + " as yet unmatched pairs. " +
-                tmp.sizeInRam() + " records in RAM.  Last sequence index: " + rec.getReferenceIndex());
+            ++index;
+            if (progress.record(rec)) {
+                log.info("Tracking " + tmp.size() + " as yet unmatched pairs. " + tmp.sizeInRam() + " records in RAM.");
             }
         }
 
         log.info("Read " + index + " records. " + tmp.size() + " pairs never matched.");
-        sam.close();
+        iterator.close();
 
         // Tell these collections to free up memory if possible.
         this.pairSort.doneAdding();
@@ -364,7 +492,7 @@ public class MarkDuplicates extends AbstractDuplicateFindingAlgorithm {
             final List<SAMReadGroupRecord> readGroups = header.getReadGroups();
 
             if (rg != null && readGroups != null) {
-                for (SAMReadGroupRecord readGroup : readGroups) {
+                for (final SAMReadGroupRecord readGroup : readGroups) {
                     if (readGroup.getReadGroupId().equals(rg)) break;
                     else ends.readGroup++;
                 }
@@ -440,7 +568,7 @@ public class MarkDuplicates extends AbstractDuplicateFindingAlgorithm {
     private void generateDuplicateIndexes() {
         final int maxInMemory = (int) ((Runtime.getRuntime().maxMemory() * 0.25) / SortingLongCollection.SIZEOF);
         log.info("Will retain up to " + maxInMemory + " duplicate indices before spilling to disk.");
-        this.duplicateIndexes = new SortingLongCollection(maxInMemory, TMP_DIR);
+        this.duplicateIndexes = new SortingLongCollection(maxInMemory, TMP_DIR.toArray(new File[TMP_DIR.size()]));
 
         ReadEnds firstOfNextChunk = null;
         final List<ReadEnds> nextChunk  = new ArrayList<ReadEnds>(200);
@@ -554,7 +682,7 @@ public class MarkDuplicates extends AbstractDuplicateFindingAlgorithm {
         final boolean[] opticalDuplicateFlags = findOpticalDuplicates(list, OPTICAL_DUPLICATE_PIXEL_DISTANCE);
 
         int opticalDuplicates = 0;
-        for (boolean b: opticalDuplicateFlags) if (b) ++opticalDuplicates;
+        for (final boolean b: opticalDuplicateFlags) if (b) ++opticalDuplicates;
         if (opticalDuplicates > 0) {
             this.opticalDupesByLibraryId.increment(list.get(0).libraryId, opticalDuplicates);
         }
@@ -605,4 +733,38 @@ public class MarkDuplicates extends AbstractDuplicateFindingAlgorithm {
             return retval;
         }
     }
+
+    static class PgIdGenerator {
+        private int recordCounter;
+
+        private final Set<String> idsThatAreAlreadyTaken = new HashSet<String>();
+
+        PgIdGenerator(final SAMFileHeader header) {
+            for (final SAMProgramRecord pgRecord : header.getProgramRecords()) {
+                idsThatAreAlreadyTaken.add(pgRecord.getProgramGroupId());
+            }
+            recordCounter = idsThatAreAlreadyTaken.size();
+        }
+
+        String getNonCollidingId(final String recordId) {
+            if(!idsThatAreAlreadyTaken.contains(recordId)) {
+                // don't remap 1st record. If there are more records
+                // with this id, they will be remapped in the 'else'.
+                idsThatAreAlreadyTaken.add(recordId);
+                ++recordCounter;
+                return recordId;
+            } else {
+                String newId;
+                // Below we tack on one of roughly 1.7 million possible 4 digit base36 at random. We do this because
+                // our old process of just counting from 0 upward and adding that to the previous id led to 1000s of
+                // calls idsThatAreAlreadyTaken.contains() just to resolve 1 collision when merging 1000s of similarly
+                // processed bams.
+                while(idsThatAreAlreadyTaken.contains(newId = recordId + "." + SamFileHeaderMerger.positiveFourDigitBase36Str(recordCounter++)));
+
+                idsThatAreAlreadyTaken.add( newId );
+                return newId;
+            }
+
+        }
+    }
 }
