@@ -5,6 +5,7 @@ import net.sf.picard.io.IoUtil;
 import net.sf.picard.util.Log;
 import net.sf.picard.util.PeekableIterator;
 import net.sf.samtools.*;
+import net.sf.samtools.SAMFileHeader.SortOrder;
 import net.sf.samtools.util.CloseableIterator;
 import net.sf.samtools.util.DelegatingIterator;
 import net.sf.samtools.util.SortingCollection;
@@ -28,7 +29,6 @@ public class SamAlignmentMerger extends AbstractAlignmentMerger {
     private final List<File> alignedSamFile;
     private final List<File> read1AlignedSamFile;
     private final List<File> read2AlignedSamFile;
-    private final boolean pairedRun;
     private final int maxGaps;
     private boolean forceSort = false;
 
@@ -65,7 +65,11 @@ public class SamAlignmentMerger extends AbstractAlignmentMerger {
      *                            alignedSamFile is not.
      * @param expectedOrientations A List of SamPairUtil.PairOrientations that are expected for
      *                          aligned pairs.  Used to determine the properPair flag.
-     *
+     * @param sortOrder           The order in which the merged records should be output.  If null,
+     *                            output will be coordinate-sorted
+     * @param primaryAlignmentSelectionStrategy How to handle multiple alignments for a fragment or read pair,
+     *                                          in which none are primary, or more than one is marked primary
+     *                                          by the aligner.
      */
     public SamAlignmentMerger (final File unmappedBamFile, final File targetBamFile, final File referenceFasta,
                  final SAMProgramRecord programRecord, final boolean clipAdapters, final boolean bisulfiteSequence,
@@ -73,11 +77,13 @@ public class SamAlignmentMerger extends AbstractAlignmentMerger {
                  final List<File> alignedSamFile, final int maxGaps, final List<String> attributesToRetain,
                  final Integer read1BasesTrimmed, final Integer read2BasesTrimmed,
                  final List<File> read1AlignedSamFile, final List<File> read2AlignedSamFile,
-                 final List<SamPairUtil.PairOrientation> expectedOrientations) {
+                 final List<SamPairUtil.PairOrientation> expectedOrientations,
+                 final SortOrder sortOrder,
+                 final PrimaryAlignmentSelectionStrategy primaryAlignmentSelectionStrategy) {
 
         super(unmappedBamFile, targetBamFile, referenceFasta, clipAdapters, bisulfiteSequence,
               alignedReadsOnly, programRecord, attributesToRetain, read1BasesTrimmed,
-              read2BasesTrimmed, expectedOrientations);
+              read2BasesTrimmed, expectedOrientations, sortOrder, primaryAlignmentSelectionStrategy);
 
         if ((alignedSamFile == null || alignedSamFile.size() == 0) &&
             (read1AlignedSamFile == null || read1AlignedSamFile.size() == 0 ||
@@ -87,15 +93,15 @@ public class SamAlignmentMerger extends AbstractAlignmentMerger {
         }
 
         if (alignedSamFile != null) {
-            for (File f : alignedSamFile) {
+            for (final File f : alignedSamFile) {
                 IoUtil.assertFileIsReadable(f);
             }
         }
         else {
-            for (File f : read1AlignedSamFile) {
+            for (final File f : read1AlignedSamFile) {
                 IoUtil.assertFileIsReadable(f);
             }
-            for (File f : read2AlignedSamFile) {
+            for (final File f : read2AlignedSamFile) {
                 IoUtil.assertFileIsReadable(f);
             }
         }
@@ -103,13 +109,12 @@ public class SamAlignmentMerger extends AbstractAlignmentMerger {
         this.alignedSamFile = alignedSamFile;
         this.read1AlignedSamFile = read1AlignedSamFile;
         this.read2AlignedSamFile = read2AlignedSamFile;
-        this.pairedRun = pairedRun;
         this.maxGaps = maxGaps;
         if (programRecord == null) {
-            File tmpFile = this.alignedSamFile != null && this.alignedSamFile.size() > 0
+            final File tmpFile = this.alignedSamFile != null && this.alignedSamFile.size() > 0
                     ? this.alignedSamFile.get(0)
                     : this.read1AlignedSamFile.get(0);
-            SAMFileReader tmpReader = new SAMFileReader(tmpFile);
+            final SAMFileReader tmpReader = new SAMFileReader(tmpFile);
             tmpReader.setValidationStringency(SAMFileReader.ValidationStringency.SILENT);
             if (tmpReader.getFileHeader().getProgramRecords().size() == 1) {
                 setProgramRecord(tmpReader.getFileHeader().getProgramRecords().get(0));
@@ -119,9 +124,9 @@ public class SamAlignmentMerger extends AbstractAlignmentMerger {
         // If not null, the program record was already added in the superclass.  DO NOT RE-ADD!
 
         if (getProgramRecord() != null) {
-            SAMFileReader tmp = new SAMFileReader(unmappedBamFile);
+            final SAMFileReader tmp = new SAMFileReader(unmappedBamFile);
             try {
-                for (SAMProgramRecord pg : tmp.getFileHeader().getProgramRecords()) {
+                for (final SAMProgramRecord pg : tmp.getFileHeader().getProgramRecords()) {
                     if (pg.getId().equals(getProgramRecord().getId())) {
                         throw new PicardException("Program Record ID already in use in unmapped BAM file.");
                     }
@@ -134,6 +139,8 @@ public class SamAlignmentMerger extends AbstractAlignmentMerger {
         log.info("Processing SAM file(s): " + alignedSamFile != null ? alignedSamFile : read1AlignedSamFile + "," + read2AlignedSamFile);
     }
 
+
+
     /**
      * Merges the alignment from the map file with the non-aligned records from the source BAM file.
      * Overrides mergeAlignment in AbstractAlignmentMerger.  Tries first to proceed on the assumption
@@ -156,20 +163,20 @@ public class SamAlignmentMerger extends AbstractAlignmentMerger {
      */
     protected CloseableIterator<SAMRecord> getQuerynameSortedAlignedRecords() {
 
-        CloseableIterator<SAMRecord> mergingIterator;
-        SAMFileHeader header;
+        final CloseableIterator<SAMRecord> mergingIterator;
+        final SAMFileHeader header;
 
         // When the alignment records, including both ends of a pair, are in SAM files
         if (alignedSamFile != null && alignedSamFile.size() > 0) {
-            List<SAMFileHeader> headers = new ArrayList<SAMFileHeader>(alignedSamFile.size());
-            List<SAMFileReader> readers = new ArrayList<SAMFileReader>(alignedSamFile.size());
-            for (File f : this.alignedSamFile) {
-                SAMFileReader r = new SAMFileReader(f);
+            final List<SAMFileHeader> headers = new ArrayList<SAMFileHeader>(alignedSamFile.size());
+            final List<SAMFileReader> readers = new ArrayList<SAMFileReader>(alignedSamFile.size());
+            for (final File f : this.alignedSamFile) {
+                final SAMFileReader r = new SAMFileReader(f);
                 headers.add(r.getFileHeader());
                 readers.add(r);
             }
 
-            SamFileHeaderMerger headerMerger = new SamFileHeaderMerger(SAMFileHeader.SortOrder.coordinate, headers, false);
+            final SamFileHeaderMerger headerMerger = new SamFileHeaderMerger(SAMFileHeader.SortOrder.coordinate, headers, false);
 
             mergingIterator = new MergingSamRecordIterator(headerMerger, readers, true);
             header = headerMerger.getMergedHeader();
@@ -202,7 +209,7 @@ public class SamAlignmentMerger extends AbstractAlignmentMerger {
         log.info("Finished reading " + count + " total records from alignment SAM/BAM.");
 
         mergingIterator.close();
-        return new DelegatingIterator(alignmentSorter.iterator()) {
+        return new DelegatingIterator<SAMRecord>(alignmentSorter.iterator()) {
             @Override
             public void close() {
                 super.close();
@@ -211,30 +218,67 @@ public class SamAlignmentMerger extends AbstractAlignmentMerger {
         };
     }
 
+    private class SuffixTrimingSamRecordIterator implements CloseableIterator<SAMRecord> {
+        private final CloseableIterator<SAMRecord> underlyingIterator;
+        private final String suffixToTrim;
+
+        private SuffixTrimingSamRecordIterator(final CloseableIterator<SAMRecord> underlyingIterator, final String suffixToTrim) {
+            this.underlyingIterator = underlyingIterator;
+            this.suffixToTrim = suffixToTrim;
+        }
+
+        @Override
+        public void close() {
+            underlyingIterator.close();
+        }
+
+        @Override
+        public boolean hasNext() {
+            return underlyingIterator.hasNext();
+        }
+
+        @Override
+        public SAMRecord next() {
+            final SAMRecord rec = underlyingIterator.next();
+            final String readName = rec.getReadName();
+            if (readName.endsWith(suffixToTrim)) {
+                rec.setReadName(readName.substring(0, readName.length() - suffixToTrim.length()));
+            }
+            return rec;
+        }
+
+        @Override
+        public void remove() {
+            underlyingIterator.remove();
+        }
+    }
+
     private class SeparateEndAlignmentIterator implements CloseableIterator<SAMRecord> {
 
         private final PeekableIterator<SAMRecord> read1Iterator;
         private final PeekableIterator<SAMRecord> read2Iterator;
         private final SAMFileHeader header;
 
-        public SeparateEndAlignmentIterator(List<File> read1Alignments, List<File> read2Alignments) {
+        public SeparateEndAlignmentIterator(final List<File> read1Alignments, final List<File> read2Alignments) {
             final List<SAMFileHeader> headers = new ArrayList<SAMFileHeader>();
             final List<SAMFileReader> read1 = new ArrayList<SAMFileReader>(read1Alignments.size());
             final List<SAMFileReader> read2 = new ArrayList<SAMFileReader>(read2Alignments.size());
-            for (File f : read1Alignments) {
-                SAMFileReader r = new SAMFileReader(f);
+            for (final File f : read1Alignments) {
+                final SAMFileReader r = new SAMFileReader(f);
                 headers.add(r.getFileHeader());
                 read1.add(r);
             }
-            for (File f : read2Alignments) {
-                SAMFileReader r = new SAMFileReader(f);
+            for (final File f : read2Alignments) {
+                final SAMFileReader r = new SAMFileReader(f);
                 headers.add(r.getFileHeader());
                 read2.add(r);
             }
 
             final SamFileHeaderMerger headerMerger = new SamFileHeaderMerger(SAMFileHeader.SortOrder.coordinate, headers, false);
-            read1Iterator = new PeekableIterator(new MergingSamRecordIterator(headerMerger, read1, true));
-            read2Iterator = new PeekableIterator(new MergingSamRecordIterator(headerMerger, read2, true));
+            read1Iterator = new PeekableIterator<SAMRecord>(
+                    new SuffixTrimingSamRecordIterator(new MergingSamRecordIterator(headerMerger, read1, true), "/1"));
+            read2Iterator = new PeekableIterator<SAMRecord>(
+                    new SuffixTrimingSamRecordIterator(new MergingSamRecordIterator(headerMerger, read2, true), "/2"));
 
             header = headerMerger.getMergedHeader();
         }
@@ -270,7 +314,7 @@ public class SamAlignmentMerger extends AbstractAlignmentMerger {
 
         public SAMFileHeader getHeader() { return this.header; }
 
-        private SAMRecord setPairFlags(SAMRecord sam, boolean firstOfPair) {
+        private SAMRecord setPairFlags(final SAMRecord sam, final boolean firstOfPair) {
             sam.setReadPairedFlag(true);
             sam.setFirstOfPairFlag(firstOfPair);
             sam.setSecondOfPairFlag(!firstOfPair);
@@ -282,10 +326,10 @@ public class SamAlignmentMerger extends AbstractAlignmentMerger {
      * For now, we only ignore those alignments that have more than <code>maxGaps</code> insertions
      * or deletions.
      */
-    protected boolean ignoreAlignment(SAMRecord sam) {
+    protected boolean ignoreAlignment(final SAMRecord sam) {
         if (maxGaps == -1) return false;
         int gaps = 0;
-        for (CigarElement el : sam.getCigar().getCigarElements()) {
+        for (final CigarElement el : sam.getCigar().getCigarElements()) {
             if (el.getOperator() == CigarOperator.I || el.getOperator() == CigarOperator.D ) {
                 gaps++;
             }
