@@ -30,8 +30,7 @@ import net.sf.picard.metrics.MetricsFile;
 import net.sf.picard.reference.ReferenceSequence;
 import net.sf.picard.reference.ReferenceSequenceFile;
 import net.sf.picard.reference.ReferenceSequenceFileWalker;
-import net.sf.picard.util.Histogram;
-import net.sf.picard.util.Log;
+import net.sf.picard.util.*;
 import net.sf.samtools.*;
 import net.sf.samtools.SAMFileReader.ValidationStringency;
 import net.sf.samtools.SAMValidationError.Type;
@@ -47,15 +46,15 @@ import java.util.*;
  * <li>checks sam file header for read groups</li>
  * <li>for each sam record
  * <ul>
- *     <li>reports error detected by SAMRecord.isValid()</li>
- *     <li>validates NM (nucleotide differences) exists and matches reality</li>
- *     <li>validates mate fields agree with data in the mate record</li>
+ * <li>reports error detected by SAMRecord.isValid()</li>
+ * <li>validates NM (nucleotide differences) exists and matches reality</li>
+ * <li>validates mate fields agree with data in the mate record</li>
  * </ul>
  * </li>
  * </ul>
  *
- * @see SAMRecord#isValid()
  * @author Doug Voet
+ * @see SAMRecord#isValid()
  */
 public class SamFileValidator {
     private Histogram<Type> errorsByType = new Histogram<Type>();
@@ -79,7 +78,9 @@ public class SamFileValidator {
         this.maxTempFiles = maxTempFiles;
     }
 
-    /** Sets one or more error types that should not be reported on. */
+    /**
+     * Sets one or more error types that should not be reported on.
+     */
     public void setErrorsToIgnore(final Collection<Type> types) {
         if (!types.isEmpty()) {
             this.errorsToIgnore = EnumSet.copyOf(types);
@@ -92,7 +93,7 @@ public class SamFileValidator {
 
     /**
      * Outputs validation summary report to out.
-     * 
+     *
      * @param samReader records to validate
      * @param reference if null, NM tag validation is skipped
      * @return boolean  true if there are no validation errors, otherwise false
@@ -122,10 +123,10 @@ public class SamFileValidator {
 
     /**
      * Outputs validation error details to out.
-     * 
+     *
      * @param samReader records to validate
      * @param reference if null, NM tag validation is skipped
-     * processing will stop after this threshold has been reached
+     *                  processing will stop after this threshold has been reached
      * @return boolean  true if there are no validation errors, otherwise false
      */
     public boolean validateSamFileVerbose(final SAMFileReader samReader, final ReferenceSequenceFile reference) {
@@ -173,16 +174,16 @@ public class SamFileValidator {
             samReader.setValidationStringency(ValidationStringency.SILENT);
             validateHeader(samReader.getFileHeader());
             orderChecker = new SAMSortOrderChecker(samReader.getFileHeader().getSortOrder());
-            validateSamRecords(samReader);
+            validateSamRecordsAndQualityFormat(samReader, samReader.getFileHeader());
             validateUnmatchedPairs();
-            if (validateIndex){
+            if (validateIndex) {
                 try {
-                   BamIndexValidator.exhaustivelyTestIndex(samReader);
-                } catch (Exception e){
+                    BamIndexValidator.exhaustivelyTestIndex(samReader);
+                } catch (Exception e) {
                     addError(new SAMValidationError(Type.INVALID_INDEX_FILE_POINTER, e.getMessage(), null));
                 }
             }
-            
+
             if (errorsByType.isEmpty()) {
                 out.println("No errors found");
             }
@@ -201,7 +202,7 @@ public class SamFileValidator {
             // For the coordinate-sorted map, need to detect mate pairs in which the mateReferenceIndex on one end
             // does not match the readReference index on the other end, so the pairs weren't united and validated.
             inMemoryPairMap = new InMemoryPairEndInfoMap();
-            CloseableIterator<Map.Entry<String, PairEndInfo>> it = ((CoordinateSortedPairEndInfoMap)pairEndInfoByName).iterator();
+            CloseableIterator<Map.Entry<String, PairEndInfo>> it = ((CoordinateSortedPairEndInfoMap) pairEndInfoByName).iterator();
             while (it.hasNext()) {
                 Map.Entry<String, PairEndInfo> entry = it.next();
                 PairEndInfo pei = inMemoryPairMap.remove(entry.getValue().readReferenceIndex, entry.getKey());
@@ -218,20 +219,30 @@ public class SamFileValidator {
             }
             it.close();
         } else {
-            inMemoryPairMap = (InMemoryPairEndInfoMap)pairEndInfoByName;
+            inMemoryPairMap = (InMemoryPairEndInfoMap) pairEndInfoByName;
         }
         // At this point, everything in InMemoryMap is a read marked as a pair, for which a mate was not found.
-        for (final Map.Entry<String, PairEndInfo> entry: inMemoryPairMap) {
+        for (final Map.Entry<String, PairEndInfo> entry : inMemoryPairMap) {
             addError(new SAMValidationError(Type.MATE_NOT_FOUND, "Mate not found for paired read", entry.getKey()));
         }
     }
 
-    private void validateSamRecords(final Iterable<SAMRecord> samRecords) {
-        long recordNumber = 1;
+    /**
+     * SAM record and quality format validations are combined into a single method because validation must be completed
+     * in only a single pass of the SamRecords (because a SamReader's iterator() method may not return the same
+     * records on a subsequent call).
+     */
+    private void validateSamRecordsAndQualityFormat(final Iterable<SAMRecord> samRecords, final SAMFileHeader header) {
         SAMRecordIterator iter = (SAMRecordIterator) samRecords.iterator();
+        final ProgressLogger progress = new ProgressLogger(log, 10000000, "Validated Read");
+        final QualityEncodingDetector qualityDetector = new QualityEncodingDetector();
         try {
-            while(iter.hasNext()){
-                SAMRecord record = iter.next();
+            while (iter.hasNext()) {
+                final SAMRecord record = iter.next();
+
+                qualityDetector.add(record);
+
+                final long recordNumber = progress.getCount() + 1;
                 final Collection<SAMValidationError> errors = record.isValid();
                 if (errors != null) {
                     for (final SAMValidationError error : errors) {
@@ -239,9 +250,10 @@ public class SamFileValidator {
                         addError(error);
                     }
                 }
-                
+
                 validateMateFields(record, recordNumber);
                 validateSortOrder(record, recordNumber);
+                validateReadGroup(record, header);
                 final boolean cigarIsValid = validateCigar(record, recordNumber);
                 if (cigarIsValid) {
                     validateNmTag(record, recordNumber);
@@ -253,16 +265,25 @@ public class SamFileValidator {
                     sequenceDictionaryEmptyAndNoWarningEmitted = false;
 
                 }
-                recordNumber++;
-                if (recordNumber % 10000000 == 0) {
-                    log.info(recordNumber + " reads validated.");
+                progress.record(record);
+            }
+
+            try {
+                if (progress.getCount() > 0) { // Avoid exception being thrown as a result of no qualities being read
+                    final FastqQualityFormat format = qualityDetector.generateBestGuess(QualityEncodingDetector.FileContext.SAM);
+                    if (format != FastqQualityFormat.Standard) {
+                        addError(new SAMValidationError(Type.INVALID_QUALITY_FORMAT, String.format("Detected %s quality score encoding, but expected %s.", format, FastqQualityFormat.Standard), null));
+                    }
                 }
+            } catch (PicardException e) {
+                addError(new SAMValidationError(Type.INVALID_QUALITY_FORMAT, e.getMessage(), null));
             }
         } catch (SAMFormatException e) {
             // increment record number because the iterator behind the SAMFileReader
             // reads one record ahead so we will get this failure one record ahead
-            out.println("SAMFormatException on record " + ++recordNumber);
-            throw new PicardException("SAMFormatException on record " + recordNumber, e);
+            final String msg = "SAMFormatException on record " + progress.getCount() + 1;
+            out.println(msg);
+            throw new PicardException(msg, e);
         } catch (FileTruncatedException e) {
             addError(new SAMValidationError(Type.TRUNCATED_FILE, "File is truncated", null));
         } finally {
@@ -270,6 +291,18 @@ public class SamFileValidator {
         }
     }
 
+    private void validateReadGroup(final SAMRecord record, final SAMFileHeader header) {
+        SAMReadGroupRecord rg = record.getReadGroup();
+        if (rg == null) {
+            addError(new SAMValidationError(Type.RECORD_MISSING_READ_GROUP,
+                    "A record is missing a read group", record.getReadName()));
+        } else if (header.getReadGroup(rg.getId()) == null) {
+            addError(new SAMValidationError(Type.READ_GROUP_NOT_FOUND,
+                    "A record has a read group not found in the header: ",
+                    record.getReadName() + ", " + rg.getReadGroupId()));
+        }
+    }
+
     /**
      * Report error if a tag value is a Long.
      */
@@ -284,7 +317,7 @@ public class SamFileValidator {
     }
 
     private void validateSecondaryBaseCalls(final SAMRecord record, final long recordNumber) {
-        final String e2 = (String)record.getAttribute(SAMTag.E2.name());
+        final String e2 = (String) record.getAttribute(SAMTag.E2.name());
         if (e2 != null) {
             if (e2.length() != record.getReadLength()) {
                 addError(new SAMValidationError(Type.MISMATCH_READ_LENGTH_AND_E2_LENGTH,
@@ -300,13 +333,13 @@ public class SamFileValidator {
                 if (SequenceUtil.basesEqual(bases[i], secondaryBases[i])) {
                     addError(new SAMValidationError(Type.E2_BASE_EQUALS_PRIMARY_BASE,
                             String.format("Secondary base call  (%c) == primary base call (%c)",
-                                    (char)secondaryBases[i], (char)bases[i]),
+                                    (char) secondaryBases[i], (char) bases[i]),
                             record.getReadName(), recordNumber));
                     break;
                 }
             }
         }
-        final String u2 = (String)record.getAttribute(SAMTag.U2.name());
+        final String u2 = (String) record.getAttribute(SAMTag.U2.name());
         if (u2 != null && u2.length() != record.getReadLength()) {
             addError(new SAMValidationError(Type.MISMATCH_READ_LENGTH_AND_U2_LENGTH,
                     String.format("U2 tag length (%d) != read length (%d)", u2.length(), record.getReadLength()),
@@ -337,18 +370,18 @@ public class SamFileValidator {
         final SAMRecord prev = orderChecker.getPreviousRecord();
         if (!orderChecker.isSorted(record)) {
             addError(new SAMValidationError(
-                    Type.RECORD_OUT_OF_ORDER, 
+                    Type.RECORD_OUT_OF_ORDER,
                     String.format(
                             "The record is out of [%s] order, prior read name [%s], prior coodinates [%d:%d]",
                             record.getHeader().getSortOrder().name(),
                             prev.getReadName(),
                             prev.getReferenceIndex(),
                             prev.getAlignmentStart()),
-                    record.getReadName(), 
+                    record.getReadName(),
                     recordNumber));
         }
     }
-    
+
     private void init(final ReferenceSequenceFile reference, final SAMFileHeader header) {
         if (header.getSortOrder() == SAMFileHeader.SortOrder.coordinate) {
             this.pairEndInfoByName = new CoordinateSortedPairEndInfoMap();
@@ -371,8 +404,8 @@ public class SamFileValidator {
             final Integer tagNucleotideDiffs = record.getIntegerAttribute(ReservedTagConstants.NM);
             if (tagNucleotideDiffs == null) {
                 addError(new SAMValidationError(
-                        Type.MISSING_TAG_NM, 
-                        "NM tag (nucleotide differences) is missing", 
+                        Type.MISSING_TAG_NM,
+                        "NM tag (nucleotide differences) is missing",
                         record.getReadName(),
                         recordNumber));
             } else if (refFileWalker != null) {
@@ -382,27 +415,27 @@ public class SamFileValidator {
 
                 if (!tagNucleotideDiffs.equals(actualNucleotideDiffs)) {
                     addError(new SAMValidationError(
-                            Type.INVALID_TAG_NM, 
+                            Type.INVALID_TAG_NM,
                             "NM tag (nucleotide differences) in file [" + tagNucleotideDiffs +
-                            "] does not match reality [" + actualNucleotideDiffs + "]", 
+                                    "] does not match reality [" + actualNucleotideDiffs + "]",
                             record.getReadName(),
                             recordNumber));
                 }
             }
         }
     }
-    
+
     private void validateMateFields(final SAMRecord record, final long recordNumber) {
         if (!record.getReadPairedFlag() || record.getNotPrimaryAlignmentFlag()) {
             return;
         }
-        
+
         final PairEndInfo pairEndInfo = pairEndInfoByName.remove(record.getReferenceIndex(), record.getReadName());
         if (pairEndInfo == null) {
             pairEndInfoByName.put(record.getMateReferenceIndex(), record.getReadName(), new PairEndInfo(record, recordNumber));
         } else {
             final List<SAMValidationError> errors =
-                pairEndInfo.validateMates(new PairEndInfo(record, recordNumber), record.getReadName());
+                    pairEndInfo.validateMates(new PairEndInfo(record, recordNumber), record.getReadName());
             for (final SAMValidationError error : errors) {
                 addError(error);
             }
@@ -428,8 +461,8 @@ public class SamFileValidator {
             addError(new SAMValidationError(Type.MISSING_READ_GROUP, "Read groups is empty", null));
         }
         List<SAMProgramRecord> pgs = fileHeader.getProgramRecords();
-        for (int i = 0; i < pgs.size()-1; i++) {
-            for (int j=i+1; j < pgs.size(); j++) {
+        for (int i = 0; i < pgs.size() - 1; i++) {
+            for (int j = i + 1; j < pgs.size(); j++) {
                 if (pgs.get(i).getProgramGroupId().equals(pgs.get(j).getProgramGroupId())) {
                     addError(new SAMValidationError(Type.DUPLICATE_PROGRAM_GROUP_ID, "Duplicate " +
                             "program group id: " + pgs.get(i).getProgramGroupId(), null));
@@ -438,8 +471,8 @@ public class SamFileValidator {
         }
 
         List<SAMReadGroupRecord> rgs = fileHeader.getReadGroups();
-        for (int i = 0; i < rgs.size()-1; i++) {
-            for (int j=i+1; j < rgs.size(); j++) {
+        for (int i = 0; i < rgs.size() - 1; i++) {
+            for (int j = i + 1; j < rgs.size(); j++) {
                 if (rgs.get(i).getReadGroupId().equals(rgs.get(j).getReadGroupId())) {
                     addError(new SAMValidationError(Type.DUPLICATE_READ_GROUP_ID, "Duplicate " +
                             "read group id: " + rgs.get(i).getReadGroupId(), null));
@@ -467,7 +500,8 @@ public class SamFileValidator {
 
     /**
      * Control verbosity
-     * @param verbose True in order to emit a message per error or warning.
+     *
+     * @param verbose          True in order to emit a message per error or warning.
      * @param maxVerboseOutput If verbose, emit no more than this many messages.  Ignored if !verbose.
      */
     public void setVerbose(final boolean verbose, final int maxVerboseOutput) {
@@ -475,8 +509,13 @@ public class SamFileValidator {
         this.maxVerboseOutput = maxVerboseOutput;
     }
 
-    public boolean isBisulfiteSequenced() { return bisulfiteSequenced; }
-    public void setBisulfiteSequenced(boolean bisulfiteSequenced) { this.bisulfiteSequenced = bisulfiteSequenced; }
+    public boolean isBisulfiteSequenced() {
+        return bisulfiteSequenced;
+    }
+
+    public void setBisulfiteSequenced(boolean bisulfiteSequenced) {
+        this.bisulfiteSequenced = bisulfiteSequenced;
+    }
 
     public SamFileValidator setValidateIndex(boolean validateIndex) {
         // The SAMFileReader must also have IndexCaching enabled to have the index validated,
@@ -487,8 +526,8 @@ public class SamFileValidator {
 
     public static class ValidationMetrics extends MetricBase {
     }
-    
-    /** 
+
+    /**
      * This class is used so we don't have to store the entire SAMRecord in memory while we wait
      * to find a record's mate and also to store the record number.
      */
@@ -502,24 +541,30 @@ public class SamFileValidator {
         private final int mateReferenceIndex;
         private final boolean mateNegStrandFlag;
         private final boolean mateUnmappedFlag;
-        
+
+        private final boolean firstOfPairFlag;
+
         private final long recordNumber;
-        
+
         public PairEndInfo(final SAMRecord record, final long recordNumber) {
             this.recordNumber = recordNumber;
-            
+
             this.readAlignmentStart = record.getAlignmentStart();
             this.readNegStrandFlag = record.getReadNegativeStrandFlag();
             this.readReferenceIndex = record.getReferenceIndex();
             this.readUnmappedFlag = record.getReadUnmappedFlag();
-            
+
             this.mateAlignmentStart = record.getMateAlignmentStart();
             this.mateNegStrandFlag = record.getMateNegativeStrandFlag();
             this.mateReferenceIndex = record.getMateReferenceIndex();
             this.mateUnmappedFlag = record.getMateUnmappedFlag();
+
+            this.firstOfPairFlag = record.getFirstOfPairFlag();
         }
 
-        private PairEndInfo(int readAlignmentStart, int readReferenceIndex, boolean readNegStrandFlag, boolean readUnmappedFlag, int mateAlignmentStart, int mateReferenceIndex, boolean mateNegStrandFlag, boolean mateUnmappedFlag, long recordNumber) {
+        private PairEndInfo(int readAlignmentStart, int readReferenceIndex, boolean readNegStrandFlag, boolean readUnmappedFlag,
+                            int mateAlignmentStart, int mateReferenceIndex, boolean mateNegStrandFlag, boolean mateUnmappedFlag,
+                            boolean firstOfPairFlag, long recordNumber) {
             this.readAlignmentStart = readAlignmentStart;
             this.readReferenceIndex = readReferenceIndex;
             this.readNegStrandFlag = readNegStrandFlag;
@@ -528,6 +573,7 @@ public class SamFileValidator {
             this.mateReferenceIndex = mateReferenceIndex;
             this.mateNegStrandFlag = mateNegStrandFlag;
             this.mateUnmappedFlag = mateUnmappedFlag;
+            this.firstOfPairFlag = firstOfPairFlag;
             this.recordNumber = recordNumber;
         }
 
@@ -535,42 +581,54 @@ public class SamFileValidator {
             final List<SAMValidationError> errors = new ArrayList<SAMValidationError>();
             validateMateFields(this, mate, readName, errors);
             validateMateFields(mate, this, readName, errors);
+            // Validations that should not be repeated on both ends
+            if (this.firstOfPairFlag == mate.firstOfPairFlag) {
+                final String whichEnd = this.firstOfPairFlag ? "first" : "second";
+                errors.add(new SAMValidationError(
+                        Type.MATES_ARE_SAME_END,
+                        "Both mates are marked as " + whichEnd + " of pair",
+                        readName,
+                        this.recordNumber
+                ));
+            }
             return errors;
         }
-        
+
         private void validateMateFields(final PairEndInfo end1, final PairEndInfo end2, final String readName, final List<SAMValidationError> errors) {
             if (end1.mateAlignmentStart != end2.readAlignmentStart) {
                 errors.add(new SAMValidationError(
-                        Type.MISMATCH_MATE_ALIGNMENT_START, 
-                        "Mate alignment does not match alignment start of mate", 
+                        Type.MISMATCH_MATE_ALIGNMENT_START,
+                        "Mate alignment does not match alignment start of mate",
                         readName,
                         end1.recordNumber));
             }
             if (end1.mateNegStrandFlag != end2.readNegStrandFlag) {
                 errors.add(new SAMValidationError(
-                        Type.MISMATCH_FLAG_MATE_NEG_STRAND, 
-                        "Mate negative strand flag does not match read negative strand flag of mate", 
+                        Type.MISMATCH_FLAG_MATE_NEG_STRAND,
+                        "Mate negative strand flag does not match read negative strand flag of mate",
                         readName,
                         end1.recordNumber));
             }
             if (end1.mateReferenceIndex != end2.readReferenceIndex) {
                 errors.add(new SAMValidationError(
-                        Type.MISMATCH_MATE_REF_INDEX, 
-                        "Mate reference index (MRNM) does not match reference index of mate", 
+                        Type.MISMATCH_MATE_REF_INDEX,
+                        "Mate reference index (MRNM) does not match reference index of mate",
                         readName,
                         end1.recordNumber));
             }
             if (end1.mateUnmappedFlag != end2.readUnmappedFlag) {
                 errors.add(new SAMValidationError(
-                        Type.MISMATCH_FLAG_MATE_UNMAPPED, 
-                        "Mate unmapped flag does not match read unmapped flag of mate", 
+                        Type.MISMATCH_FLAG_MATE_UNMAPPED,
+                        "Mate unmapped flag does not match read unmapped flag of mate",
                         readName,
                         end1.recordNumber));
             }
         }
     }
-    
-    /** Thrown in addError indicating that maxVerboseOutput has been exceeded and processing should stop */
+
+    /**
+     * Thrown in addError indicating that maxVerboseOutput has been exceeded and processing should stop
+     */
     private static class MaxOutputExceededException extends PicardException {
         MaxOutputExceededException() {
             super("maxVerboseOutput exceeded.");
@@ -579,7 +637,9 @@ public class SamFileValidator {
 
     interface PairEndInfoMap extends Iterable<Map.Entry<String, PairEndInfo>> {
         void put(int mateReferenceIndex, String key, PairEndInfo value);
+
         PairEndInfo remove(int mateReferenceIndex, String key);
+
         CloseableIterator<Map.Entry<String, PairEndInfo>> iterator();
     }
 
@@ -592,7 +652,7 @@ public class SamFileValidator {
         }
 
         public PairEndInfo remove(int mateReferenceIndex, String key) {
-            return onDiskMap.remove(mateReferenceIndex, key); 
+            return onDiskMap.remove(mateReferenceIndex, key);
         }
 
         public CloseableIterator<Map.Entry<String, PairEndInfo>> iterator() {
@@ -603,8 +663,13 @@ public class SamFileValidator {
             private DataInputStream in;
             private DataOutputStream out;
 
-            public void setOutputStream(final OutputStream os) { this.out = new DataOutputStream(os); }
-            public void setInputStream(final InputStream is) { this.in = new DataInputStream(is); }
+            public void setOutputStream(final OutputStream os) {
+                this.out = new DataOutputStream(os);
+            }
+
+            public void setInputStream(final InputStream is) {
+                this.in = new DataInputStream(is);
+            }
 
             public void encode(String key, PairEndInfo record) {
                 try {
@@ -617,6 +682,7 @@ public class SamFileValidator {
                     out.writeInt(record.mateReferenceIndex);
                     out.writeBoolean(record.mateNegStrandFlag);
                     out.writeBoolean(record.mateUnmappedFlag);
+                    out.writeBoolean(record.firstOfPairFlag);
                     out.writeLong(record.recordNumber);
                 } catch (IOException e) {
                     throw new PicardException("Error spilling PairInfo to disk", e);
@@ -636,8 +702,12 @@ public class SamFileValidator {
                     final boolean mateNegStrandFlag = in.readBoolean();
                     final boolean mateUnmappedFlag = in.readBoolean();
 
+                    final boolean firstOfPairFlag = in.readBoolean();
+
                     final long recordNumber = in.readLong();
-                    final PairEndInfo rec = new PairEndInfo(readAlignmentStart, readReferenceIndex, readNegStrandFlag, readUnmappedFlag, mateAlignmentStart, mateReferenceIndex, mateNegStrandFlag, mateUnmappedFlag, recordNumber);
+                    final PairEndInfo rec = new PairEndInfo(readAlignmentStart, readReferenceIndex, readNegStrandFlag,
+                            readUnmappedFlag, mateAlignmentStart, mateReferenceIndex, mateNegStrandFlag, mateUnmappedFlag,
+                            firstOfPairFlag, recordNumber);
                     return new AbstractMap.SimpleEntry(key, rec);
                 } catch (IOException e) {
                     throw new PicardException("Error reading PairInfo from disk", e);
