@@ -24,7 +24,10 @@
 package net.sf.samtools;
 
 
+import net.sf.samtools.seekablestream.SeekableBufferedStream;
 import net.sf.samtools.util.*;
+import net.sf.samtools.seekablestream.SeekableHTTPStream;
+import net.sf.samtools.seekablestream.SeekableStream;
 
 import java.io.*;
 import java.util.Arrays;
@@ -56,6 +59,7 @@ public class SAMFileReader implements Iterable<SAMRecord>, Closeable {
 
     private boolean mIsBinary = false;
     private BAMIndex mIndex = null;
+    private SAMRecordFactory samRecordFactory = new DefaultSAMRecordFactory();
     private ReaderImplementation mReader = null;
 
     private File samFile = null;
@@ -89,6 +93,7 @@ public class SAMFileReader implements Iterable<SAMRecord>, Closeable {
         abstract void enableIndexCaching(final boolean enabled);
         abstract void enableIndexMemoryMapping(final boolean enabled);
         abstract void enableCrcChecking(final boolean enabled);
+        abstract void setSAMRecordFactory(final SAMRecordFactory factory);
         abstract boolean hasIndex();
         abstract BAMIndex getIndex();
         abstract SAMFileHeader getFileHeader();
@@ -190,6 +195,10 @@ public class SAMFileReader implements Iterable<SAMRecord>, Closeable {
         init(strm, indexFile, eagerDecode, defaultValidationStringency);
     }
 
+    public SAMFileReader(final SeekableStream strm, final SeekableStream indexStream, final boolean eagerDecode) {
+        init(strm, indexStream, eagerDecode, defaultValidationStringency);
+    }
+
     public void close() {
         if (mReader != null) {
             mReader.close();
@@ -236,6 +245,14 @@ public class SAMFileReader implements Iterable<SAMRecord>, Closeable {
         this.mReader.enableCrcChecking(enabled);
     }
 
+    /**
+     * Override the default SAMRecordFactory class used to instantiate instances of SAMRecord and BAMRecord.
+     */
+    public void setSAMRecordFactory(final SAMRecordFactory factory) {
+        this.samRecordFactory = factory;
+        this.mReader.setSAMRecordFactory(factory);
+    }
+
     /**
      * @return True if this is a BAM reader.
      */
@@ -418,6 +435,11 @@ public class SAMFileReader implements Iterable<SAMRecord>, Closeable {
      * mate information.  This method iterates over the SAM file, so there may not be an unclosed
      * iterator on the SAM file when this method is called.
      *
+     * Note that it is not possible to call queryMate when iterating over the SAMFileReader, because queryMate
+     * requires its own iteration, and there cannot be two simultaneous iterations on the same SAMFileReader.  The
+     * work-around is to open a second SAMFileReader on the same input file, and call queryMate on the second
+     * reader.
+     *
      * @param rec Record for which mate is sought.  Must be a paired read.
      * @return rec's mate, or null if it cannot be found.
      */
@@ -465,16 +487,13 @@ public class SAMFileReader implements Iterable<SAMRecord>, Closeable {
     }
 
 
-
     private void init(final SeekableStream strm, final File indexFile, final boolean eagerDecode,
                       final ValidationStringency validationStringency) {
 
         try {
-            // Its too expensive to examine the remote file to determine type.
-            // Rely on file extension.
-            if (strm.getSource() == null || strm.getSource().toLowerCase().endsWith(".bam")) {
+            if (streamLooksLikeBam(strm)) {
                 mIsBinary = true;
-                mReader = new BAMFileReader(strm, indexFile, eagerDecode, validationStringency);
+                mReader = new BAMFileReader(strm, indexFile, eagerDecode, validationStringency, this.samRecordFactory);
             } else {
                 throw new SAMFormatException("Unrecognized file format: " + strm);
             }
@@ -485,6 +504,28 @@ public class SAMFileReader implements Iterable<SAMRecord>, Closeable {
         }
     }
 
+    private void init(final SeekableStream strm, final SeekableStream indexStream, final boolean eagerDecode,
+                      final ValidationStringency validationStringency) {
+
+        try {
+            if (streamLooksLikeBam(strm)) {
+                mIsBinary = true;
+                mReader = new BAMFileReader(strm, indexStream, eagerDecode, validationStringency, this.samRecordFactory);
+            } else {
+                throw new SAMFormatException("Unrecognized file format: " + strm);
+            }
+            setValidationStringency(validationStringency);
+        }
+        catch (IOException e) {
+            throw new RuntimeIOException(e);
+        }
+    }
+
+    // Its too expensive to examine the remote file to determine type.
+    // Rely on file extension.
+    private boolean streamLooksLikeBam(SeekableStream strm) {
+        return strm.getSource() == null || strm.getSource().toLowerCase().endsWith(".bam");
+    }
 
     private void init(final InputStream stream, final File file, File indexFile, final boolean eagerDecode, final ValidationStringency validationStringency) {
         if (stream != null && file != null) throw new IllegalArgumentException("stream and file are mutually exclusive");
@@ -498,29 +539,31 @@ public class SAMFileReader implements Iterable<SAMRecord>, Closeable {
                 mIsBinary = true;
                 if (file == null || !file.isFile()) {
                     // Handle case in which file is a named pipe, e.g. /dev/stdin or created by mkfifo
-                    mReader = new BAMFileReader(bufferedStream, indexFile, eagerDecode, validationStringency);
+                    mReader = new BAMFileReader(bufferedStream, indexFile, eagerDecode, validationStringency, this.samRecordFactory);
                 } else {
                     bufferedStream.close();
-                    mReader = new BAMFileReader(file, indexFile, eagerDecode, validationStringency);
+                    mReader = new BAMFileReader(file, indexFile, eagerDecode, validationStringency, this.samRecordFactory);
                 }
             } else if (BlockCompressedInputStream.isValidFile(bufferedStream)) {
                 mIsBinary = false;
-                mReader = new SAMTextReader(new BlockCompressedInputStream(bufferedStream), validationStringency);
+                mReader = new SAMTextReader(new BlockCompressedInputStream(bufferedStream), validationStringency, this.samRecordFactory);
             } else if (isGzippedSAMFile(bufferedStream)) {
                 mIsBinary = false;
-                mReader = new SAMTextReader(new GZIPInputStream(bufferedStream), validationStringency);
+                mReader = new SAMTextReader(new GZIPInputStream(bufferedStream), validationStringency, this.samRecordFactory);
             } else if (isSAMFile(bufferedStream)) {
                 if (indexFile != null) {
                     bufferedStream.close();
                     throw new RuntimeException("Cannot use index file with textual SAM file");
                 }
                 mIsBinary = false;
-                mReader = new SAMTextReader(bufferedStream, file, validationStringency);
+                mReader = new SAMTextReader(bufferedStream, file, validationStringency, this.samRecordFactory);
             } else {
                 bufferedStream.close();
                 throw new SAMFormatException("Unrecognized file format");
             }
+
             setValidationStringency(validationStringency);
+            mReader.setSAMRecordFactory(this.samRecordFactory);
         }
         catch (IOException e) {
             throw new RuntimeIOException(e);
