@@ -27,7 +27,11 @@ package net.sf.samtools;
 import net.sf.samtools.util.CoordMath;
 import net.sf.samtools.util.StringUtil;
 
-import java.util.*;
+import java.lang.reflect.Array;
+import java.util.ArrayList;
+import java.util.Arrays;
+import java.util.Collections;
+import java.util.List;
 
 
 /**
@@ -37,12 +41,12 @@ import java.util.*;
  * do not necessarily mean that a read is aligned.  Those values may merely be set to force a SAMRecord
  * to appear in a certain place in the sort order.  The readUnmappedFlag must be checked to determine whether
  * or not a read is mapped.  Only if the readUnmappedFlag is false can the reference name/index and alignment start
- * be interpretted as indicating an actual alignment position.
+ * be interpreted as indicating an actual alignment position.
  *
  * Likewise, presence of mate reference name/index and mate alignment start do not necessarily mean that the
  * mate is aligned.  These may be set for an unaligned mate if the mate has been forced into a particular place
  * in the sort order per the above paragraph.  Only if the mateUnmappedFlag is false can the mate reference name/index
- * and mate alignment start be interpretted as indicating the actual alignment position of the mate.
+ * and mate alignment start be interpreted as indicating the actual alignment position of the mate.
  *
  * Note also that there are a number of getters & setters that are linked, i.e. they present different representations
  * of the same underlying data.  In these cases there is typically a representation that is preferred because it
@@ -69,6 +73,9 @@ import java.util.*;
  * Some of the get() methods return values that are mutable, due to the limitations of Java.  A caller should
  * never change the value returned by a get() method.  If you want to change the value of some attribute of a
  * SAMRecord, create a new value object and call the appropriate set() method.
+ *
+ * By default, extensive validation of SAMRecords is done when they are read.  Very limited validation is done when
+ * values are set onto SAMRecords.
  */
 public class SAMRecord implements Cloneable
 {
@@ -154,8 +161,8 @@ public class SAMRecord implements Cloneable
     private int mMateAlignmentStart = 0;
     private int mInferredInsertSize = 0;
     private SAMBinaryTagAndValue mAttributes = null;
-    private Integer mReferenceIndex = null;
-    private Integer mMateReferenceIndex = null;
+    protected Integer mReferenceIndex = null;
+    protected Integer mMateReferenceIndex = null;
     private Integer mIndexingBin = null;
 
     /**
@@ -310,6 +317,19 @@ public class SAMRecord implements Cloneable
     }
 
     public void setReferenceName(final String value) {
+        /* String.intern() is surprisingly expensive, so avoid it by looking up in sequence dictionary if possible */
+        if (NO_ALIGNMENT_REFERENCE_NAME.equals(value)) {
+            mReferenceName = NO_ALIGNMENT_REFERENCE_NAME;
+            mReferenceIndex = NO_ALIGNMENT_REFERENCE_INDEX;
+            return;
+        } else if (mHeader != null) {
+            int referenceIndex = mHeader.getSequenceIndex(value);
+            if (referenceIndex != -1) {
+                setReferenceIndex(referenceIndex);
+                return;
+            }
+        }
+        // Drop through from above if nothing done.
         mReferenceName = value.intern();
         mReferenceIndex = null;
     }
@@ -356,6 +376,19 @@ public class SAMRecord implements Cloneable
     }
 
     public void setMateReferenceName(final String mateReferenceName) {
+        /* String.intern() is surprisingly expensive, so avoid it by looking up in sequence dictionary if possible */
+        if (NO_ALIGNMENT_REFERENCE_NAME.equals(mateReferenceName)) {
+            mMateReferenceName = NO_ALIGNMENT_REFERENCE_NAME;
+            mMateReferenceIndex = NO_ALIGNMENT_REFERENCE_INDEX;
+            return;
+        } else if (mHeader != null) {
+            int referenceIndex = mHeader.getSequenceIndex(mateReferenceName);
+            if (referenceIndex != -1) {
+                setMateReferenceIndex(referenceIndex);
+                return;
+            }
+        }
+        // Drop through from above if nothing done.
         this.mMateReferenceName = mateReferenceName.intern();
         mMateReferenceIndex = null;
     }
@@ -951,6 +984,7 @@ public class SAMRecord implements Cloneable
         throw new SAMException("Value for tag " + tag + " is not a Float: " + val.getClass());
     }
 
+    /** Will work for signed byte array, unsigned byte array, or old-style hex array */
     public byte[] getByteArrayAttribute(final String tag) {
         final Object val = getAttribute(tag);
         if (val == null) return null;
@@ -960,7 +994,90 @@ public class SAMRecord implements Cloneable
         throw new SAMException("Value for tag " + tag + " is not a byte[]: " + val.getClass());
     }
 
-    protected Object getAttribute(final short tag) {
+    public byte[] getUnsignedByteArrayAttribute(final String tag) {
+        final byte[] ret = getByteArrayAttribute(tag);
+        if (ret != null) requireUnsigned(tag);
+        return ret;
+    }
+
+    /** Will work for signed byte array or old-style hex array */
+    public byte[] getSignedByteArrayAttribute(final String tag) {
+        final byte[] ret = getByteArrayAttribute(tag);
+        if (ret != null) requireSigned(tag);
+        return ret;
+    }
+
+    public short[] getUnsignedShortArrayAttribute(final String tag) {
+        final Object val = getAttribute(tag);
+        if (val == null) return null;
+        if (val instanceof short[]) {
+            requireUnsigned(tag);
+            return (short[]) val;
+        }
+        throw new SAMException("Value for tag " + tag + " is not a short[]: " + val.getClass());
+    }
+
+    public short[] getSignedShortArrayAttribute(final String tag) {
+        final Object val = getAttribute(tag);
+        if (val == null) return null;
+        if (val instanceof short[]) {
+            requireSigned(tag);
+            return (short[]) val;
+        }
+        throw new SAMException("Value for tag " + tag + " is not a short[]: " + val.getClass());
+    }
+
+    public int[] getUnsignedIntArrayAttribute(final String tag) {
+        final Object val = getAttribute(tag);
+        if (val == null) return null;
+        if (val instanceof int[]) {
+            requireUnsigned(tag);
+            return (int[]) val;
+        }
+        throw new SAMException("Value for tag " + tag + " is not a int[]: " + val.getClass());
+    }
+
+    public int[] getSignedIntArrayAttribute(final String tag) {
+        final Object val = getAttribute(tag);
+        if (val == null) return null;
+        if (val instanceof int[]) {
+            requireSigned(tag);
+            return (int[]) val;
+        }
+        throw new SAMException("Value for tag " + tag + " is not a int[]: " + val.getClass());
+    }
+
+    public float[] getFloatArrayAttribute(final String tag) {
+        final Object val = getAttribute(tag);
+        if (val != null && !(val instanceof float[])) {
+            throw new SAMException("Value for tag " + tag + " is not a float[]: " + val.getClass());
+        }
+        return (float[]) val;
+    }
+
+    /**
+     * @return True if this tag is an unsigned array, else false.
+     * @throws SAMException if the tag is not present.
+     */
+    public boolean isUnsignedArrayAttribute(final String tag) {
+        final SAMBinaryTagAndValue tmp = this.mAttributes.find(SAMTagUtil.getSingleton().makeBinaryTag(tag));
+        if (tmp != null) return tmp.isUnsignedArray();
+        throw new SAMException("Tag " + tag + " is not present in this SAMRecord");
+    }
+
+    private void requireSigned(final String tag) {
+        if (isUnsignedArrayAttribute(tag))  throw new SAMException("Value for tag " + tag + " is not signed");
+    }
+
+    private void requireUnsigned(final String tag) {
+        if (!isUnsignedArrayAttribute(tag))  throw new SAMException("Value for tag " + tag + " is not unsigned");
+    }
+
+    /**
+     * @see SAMRecord#getAttribute(java.lang.String)
+     * @param tag Binary representation of a 2-char String tag as created by SAMTagUtil.
+     */
+    public Object getAttribute(final short tag) {
         if (this.mAttributes == null) return null;
         else {
             final SAMBinaryTagAndValue tmp = this.mAttributes.find(tag);
@@ -972,7 +1089,7 @@ public class SAMRecord implements Cloneable
     /**
      * Set a named attribute onto the SAMRecord.  Passing a null value causes the attribute to be cleared.
      * @param tag two-character tag name.  See http://samtools.sourceforge.net/SAM1.pdf for standard and user-defined tags.
-     * @param value Supported types are String, Char, Integer, Float, byte[].
+     * @param value Supported types are String, Char, Integer, Float, byte[], short[]. int[], float[].
      * If value == null, tag is cleared.
      *
      * Byte and Short are allowed but discouraged.  If written to a SAM file, these will be converted to Integer,
@@ -980,25 +1097,68 @@ public class SAMRecord implements Cloneable
      *
      * Long with value between 0 and MAX_UINT is allowed for BAM but discouraged.  Attempting to write such a value
      * to SAM will cause an exception to be thrown.
+     *
+     * To set unsigned byte[], unsigned short[] or unsigned int[] (which is discouraged because of poor Java language
+     * support), setUnsignedArrayAttribute() must be used instead of this method.
+     *
+     * String values are not validated to ensure that they conform to SAM spec.
      */
     public void setAttribute(final String tag, final Object value) {
+        if (value != null && value.getClass().isArray() && Array.getLength(value) == 0) {
+            throw new IllegalArgumentException("Empty value passed for tag " + tag);
+        }
         setAttribute(SAMTagUtil.getSingleton().makeBinaryTag(tag), value);
     }
 
+    /**
+     * Because Java does not support unsigned integer types, we think it is a bad idea to encode them in SAM
+     * files.  If you must do so, however, you must call this method rather than setAttribute, because calling
+     * this method is the way to indicate that, e.g. a short array should be interpreted as unsigned shorts.
+     * @param value must be one of byte[], short[], int[]
+     */
+    public void setUnsignedArrayAttribute(final String tag, final Object value) {
+        if (!value.getClass().isArray()) {
+            throw new IllegalArgumentException("Non-array passed to setUnsignedArrayAttribute for tag " + tag);
+        }
+        if (Array.getLength(value) == 0) {
+            throw new IllegalArgumentException("Empty array passed to setUnsignedArrayAttribute for tag " + tag);
+        }
+        setAttribute(SAMTagUtil.getSingleton().makeBinaryTag(tag), value, true);
+    }
+
+    /**
+     * @see net.sf.samtools.SAMRecord#setAttribute(java.lang.String, java.lang.Object)
+     * @param tag Binary representation of a 2-char String tag as created by SAMTagUtil.
+     */
     protected void setAttribute(final short tag, final Object value) {
+        setAttribute(tag, value, false);
+    }
+
+    protected void setAttribute(final short tag, final Object value, boolean isUnsignedArray) {
         if (value != null &&
                 !(value instanceof Byte || value instanceof Short || value instanceof Integer ||
                 value instanceof String || value instanceof Character || value instanceof Float ||
-                value instanceof byte[])) {
+                value instanceof byte[] || value instanceof short[] || value instanceof int[] ||
+                        value instanceof float[])) {
             throw new SAMException("Attribute type " + value.getClass() + " not supported. Tag: " +
                     SAMTagUtil.getSingleton().makeStringTag(tag));
         }
-        // It's a new tag
         if (value == null) {
             if (this.mAttributes != null) this.mAttributes = this.mAttributes.remove(tag);
         }
         else {
-            final SAMBinaryTagAndValue tmp = new SAMBinaryTagAndValue(tag, value);
+            final SAMBinaryTagAndValue tmp;
+            if(!isUnsignedArray) {
+                tmp = new SAMBinaryTagAndValue(tag, value);
+            }
+            else {
+                if (!value.getClass().isArray() || value instanceof float[]) {
+                    throw new SAMException("Attribute type " + value.getClass() +
+                            " cannot be encoded as an unsigned array. Tag: " +
+                            SAMTagUtil.getSingleton().makeStringTag(tag));
+                }
+                tmp = new SAMBinaryTagAndUnsignedArrayValue(tag, value);
+            }
             if (this.mAttributes == null) this.mAttributes = tmp;
             else this.mAttributes = this.mAttributes.insert(tmp);
         }
@@ -1111,6 +1271,12 @@ public class SAMRecord implements Cloneable
         return -1;
     }
 
+    /**
+     *
+     * @return String representation of this.
+     * @deprecated This method is not guaranteed to return a valid SAM text representation of the SAMRecord.
+     * To get standard SAM text representation, use net.sf.samtools.SAMRecord#getSAMString().
+     */
     public String format() {
         final StringBuilder buffer = new StringBuilder();
         addField(buffer, getReadName(), null, null);
@@ -1345,7 +1511,7 @@ public class SAMRecord implements Cloneable
             }
             if (getSecondOfPairFlagUnchecked()) {
                 if (ret == null) ret = new ArrayList<SAMValidationError>();
-                ret.add(new SAMValidationError(SAMValidationError.Type.INVALID_FLAG_SECOND_OF_PAIR, "First of pair flag should not be set for unpaired read.", getReadName()));
+                ret.add(new SAMValidationError(SAMValidationError.Type.INVALID_FLAG_SECOND_OF_PAIR, "Second of pair flag should not be set for unpaired read.", getReadName()));
             }
             if (getMateReferenceIndex() != NO_ALIGNMENT_REFERENCE_INDEX) {
                 if (ret == null) ret = new ArrayList<SAMValidationError>();
@@ -1389,10 +1555,7 @@ public class SAMRecord implements Cloneable
                 if (ret == null) ret = new ArrayList<SAMValidationError>();
                 ret.add(new SAMValidationError(SAMValidationError.Type.INVALID_MAPPING_QUALITY, "MAPQ should be 0 for unmapped read.", getReadName()));
             }
-            if (getCigarLength() != 0) {
-                if (ret == null) ret = new ArrayList<SAMValidationError>();
-                ret.add(new SAMValidationError(SAMValidationError.Type.INVALID_CIGAR, "CIGAR should have zero elements for unmapped read.", getReadName()));
-            }
+            /* non-empty CIGAR on unmapped read is now allowed, because there are special reads when SAM is used to store assembly. */
 /*
             TODO: PIC-97 This validation should be enabled, but probably at this point there are too many
             BAM files that have the proper pair flag set when read or mate is unmapped.
@@ -1447,25 +1610,28 @@ public class SAMRecord implements Cloneable
             ret.addAll(errors);
         }
         if (this.getReadLength() == 0 && !this.getNotPrimaryAlignmentFlag()) {
-            String cq = (String)getAttribute(SAMTagUtil.getSingleton().CQ);
-            String cs = (String)getAttribute(SAMTagUtil.getSingleton().CS);
-            if (cq == null || cq.length() == 0 || cs == null || cs.length() == 0) {
-                if (ret == null) ret = new ArrayList<SAMValidationError>();
-                ret.add(new SAMValidationError(SAMValidationError.Type.EMPTY_READ,
-                        "Zero-length read without CS or CQ tag", getReadName()));
-            } else if (!getReadUnmappedFlag()) {
-                boolean hasIndel = false;
-                for (CigarElement cigarElement : getCigar().getCigarElements()) {
-                    if (cigarElement.getOperator() == CigarOperator.DELETION ||
-                            cigarElement.getOperator() == CigarOperator.INSERTION) {
-                        hasIndel = true;
-                        break;
-                    }
-                }
-                if (!hasIndel) {
+            Object fz = getAttribute(SAMTagUtil.getSingleton().FZ);
+            if (fz == null) {
+                String cq = (String)getAttribute(SAMTagUtil.getSingleton().CQ);
+                String cs = (String)getAttribute(SAMTagUtil.getSingleton().CS);
+                if (cq == null || cq.length() == 0 || cs == null || cs.length() == 0) {
                     if (ret == null) ret = new ArrayList<SAMValidationError>();
                     ret.add(new SAMValidationError(SAMValidationError.Type.EMPTY_READ,
-                            "Colorspace read with zero-length bases but no indel", getReadName()));
+                            "Zero-length read without FZ, CS or CQ tag", getReadName()));
+                } else if (!getReadUnmappedFlag()) {
+                    boolean hasIndel = false;
+                    for (CigarElement cigarElement : getCigar().getCigarElements()) {
+                        if (cigarElement.getOperator() == CigarOperator.DELETION ||
+                                cigarElement.getOperator() == CigarOperator.INSERTION) {
+                            hasIndel = true;
+                            break;
+                        }
+                    }
+                    if (!hasIndel) {
+                        if (ret == null) ret = new ArrayList<SAMValidationError>();
+                        ret.add(new SAMValidationError(SAMValidationError.Type.EMPTY_READ,
+                                "Colorspace read with zero-length bases but no indel", getReadName()));
+                    }
                 }
             }
         }
@@ -1474,6 +1640,15 @@ public class SAMRecord implements Cloneable
             ret.add(new SAMValidationError(SAMValidationError.Type.MISMATCH_READ_LENGTH_AND_QUALS_LENGTH,
                     "Read length does not match quals length", getReadName()));
         }
+
+        if (this.getAlignmentStart() != NO_ALIGNMENT_START && this.getIndexingBin() != null &&
+                this.computeIndexingBin() != this.getIndexingBin()) {
+            if (ret == null) ret = new ArrayList<SAMValidationError>();
+            ret.add(new SAMValidationError(SAMValidationError.Type.INVALID_INDEXING_BIN,
+                    "bin field of BAM record does not equal value computed based on alignment start and end, and length of sequence to which read is aligned",
+                    getReadName()));
+        }
+
         if (ret == null || ret.size() == 0) {
             return null;
         }
@@ -1579,5 +1754,13 @@ public class SAMRecord implements Cloneable
 
         return builder.toString();
     }
+
+    /**
+	Returns the record in the SAM line-based text format.  Fields are
+	separated by '\t' characters, and the String is terminated by '\n'.
+    */
+    public String getSAMString() {
+	return SAMTextWriter.getSAMString(this);
+    }
 }
 
