@@ -38,8 +38,18 @@ public final class SAMUtils
     private static final byte COMPRESSED_EQUAL_LOW = 0;
     private static final byte COMPRESSED_A_LOW = 1;
     private static final byte COMPRESSED_C_LOW = 2;
+    private static final byte COMPRESSED_M_LOW = 3;
     private static final byte COMPRESSED_G_LOW = 4;
+    private static final byte COMPRESSED_R_LOW = 5;
+    private static final byte COMPRESSED_S_LOW = 6;
+    private static final byte COMPRESSED_V_LOW = 7;
     private static final byte COMPRESSED_T_LOW = 8;
+    private static final byte COMPRESSED_W_LOW = 9;
+    private static final byte COMPRESSED_Y_LOW = 10;
+    private static final byte COMPRESSED_H_LOW = 11;
+    private static final byte COMPRESSED_K_LOW = 12;
+    private static final byte COMPRESSED_D_LOW = 13;
+    private static final byte COMPRESSED_B_LOW = 14;
     private static final byte COMPRESSED_N_LOW = 15;
     private static final byte COMPRESSED_EQUAL_HIGH = COMPRESSED_EQUAL_LOW << 4;
     private static final byte COMPRESSED_A_HIGH = COMPRESSED_A_LOW << 4;
@@ -47,6 +57,19 @@ public final class SAMUtils
     private static final byte COMPRESSED_G_HIGH = COMPRESSED_G_LOW << 4;
     private static final byte COMPRESSED_T_HIGH = (byte)(COMPRESSED_T_LOW << 4);
     private static final byte COMPRESSED_N_HIGH = (byte)(COMPRESSED_N_LOW << 4);
+
+    private static final byte COMPRESSED_M_HIGH = (byte)(COMPRESSED_M_LOW << 4);
+    private static final byte COMPRESSED_R_HIGH = (byte)(COMPRESSED_R_LOW << 4);
+    private static final byte COMPRESSED_S_HIGH = (byte)(COMPRESSED_S_LOW << 4);
+    private static final byte COMPRESSED_V_HIGH = (byte)(COMPRESSED_V_LOW << 4);
+    private static final byte COMPRESSED_W_HIGH = (byte)(COMPRESSED_W_LOW << 4);
+    private static final byte COMPRESSED_Y_HIGH = (byte)(COMPRESSED_Y_LOW << 4);
+    private static final byte COMPRESSED_H_HIGH = (byte)(COMPRESSED_H_LOW << 4);
+    private static final byte COMPRESSED_K_HIGH = (byte)(COMPRESSED_K_LOW << 4);
+    private static final byte COMPRESSED_D_HIGH = (byte)(COMPRESSED_D_LOW << 4);
+    private static final byte COMPRESSED_B_HIGH = (byte)(COMPRESSED_B_LOW << 4);
+
+
     public static final int MAX_PHRED_SCORE = 93;
 
     /**
@@ -117,6 +140,38 @@ public final class SAMUtils
             case 'N':
             case '.':
                 return COMPRESSED_N_LOW;
+
+            // IUPAC ambiguity codes
+            case 'M':
+            case 'm':
+                return COMPRESSED_M_LOW;
+            case 'R':
+            case 'r':
+                return COMPRESSED_R_LOW;
+            case 'S':
+            case 's':
+                return COMPRESSED_S_LOW;
+            case 'V':
+            case 'v':
+                return COMPRESSED_V_LOW;
+            case 'W':
+            case 'w':
+                return COMPRESSED_W_LOW;
+            case 'Y':
+            case 'y':
+                return COMPRESSED_Y_LOW;
+            case 'H':
+            case 'h':
+                return COMPRESSED_H_LOW;
+            case 'K':
+            case 'k':
+                return COMPRESSED_K_LOW;
+            case 'D':
+            case 'd':
+                return COMPRESSED_D_LOW;
+            case 'B':
+            case 'b':
+                return COMPRESSED_B_LOW;
             default:
                 throw new IllegalArgumentException("Bad  byte passed to charToCompressedBase: " + base);
         }
@@ -147,6 +202,38 @@ public final class SAMUtils
             case 'N':
             case '.':
                 return COMPRESSED_N_HIGH;
+
+            // IUPAC ambiguity codes
+            case 'M':
+            case 'm':
+                return COMPRESSED_M_HIGH;
+            case 'R':
+            case 'r':
+                return COMPRESSED_R_HIGH;
+            case 'S':
+            case 's':
+                return COMPRESSED_S_HIGH;
+            case 'V':
+            case 'v':
+                return COMPRESSED_V_HIGH;
+            case 'W':
+            case 'w':
+                return COMPRESSED_W_HIGH;
+            case 'Y':
+            case 'y':
+                return COMPRESSED_Y_HIGH;
+            case 'H':
+            case 'h':
+                return COMPRESSED_H_HIGH;
+            case 'K':
+            case 'k':
+                return COMPRESSED_K_HIGH;
+            case 'D':
+            case 'd':
+                return COMPRESSED_D_HIGH;
+            case 'B':
+            case 'b':
+                return COMPRESSED_B_HIGH;
             default:
                 throw new IllegalArgumentException("Bad  byte passed to charToCompressedBase: " + base);
         }
@@ -171,6 +258,20 @@ public final class SAMUtils
                 return 'T';
             case COMPRESSED_N_LOW:
                 return 'N';
+
+            // IUPAC ambiguity codes
+            case COMPRESSED_M_LOW: return 'M';
+            case COMPRESSED_R_LOW: return 'R';
+            case COMPRESSED_S_LOW: return 'S';
+            case COMPRESSED_V_LOW: return 'V';
+            case COMPRESSED_W_LOW: return 'W';
+            case COMPRESSED_Y_LOW: return 'Y';
+            case COMPRESSED_H_LOW: return 'H';
+            case COMPRESSED_K_LOW: return 'K';
+            case COMPRESSED_D_LOW: return 'D';
+            case COMPRESSED_B_LOW: return 'B';
+
+
             default:
                 throw new IllegalArgumentException("Bad  byte passed to charToCompressedBase: " + base);
         }
@@ -195,6 +296,19 @@ public final class SAMUtils
                 return 'T';
             case COMPRESSED_N_HIGH:
                 return 'N';
+
+            // IUPAC ambiguity codes
+            case COMPRESSED_M_HIGH: return 'M';
+            case COMPRESSED_R_HIGH: return 'R';
+            case COMPRESSED_S_HIGH: return 'S';
+            case COMPRESSED_V_HIGH: return 'V';
+            case COMPRESSED_W_HIGH: return 'W';
+            case COMPRESSED_Y_HIGH: return 'Y';
+            case COMPRESSED_H_HIGH: return 'H';
+            case COMPRESSED_K_HIGH: return 'K';
+            case COMPRESSED_D_HIGH: return 'D';
+            case COMPRESSED_B_HIGH: return 'B';
+
             default:
                 throw new IllegalArgumentException("Bad  byte passed to charToCompressedBase: " + base);
         }
@@ -320,7 +434,7 @@ public final class SAMUtils
      * @param validationErrors List of errors to report, or null if there are no errors.
      * @param samRecordIndex Record number of the SAMRecord corresponding to the validation errors, or -1 if
      * the record number is not known.
-     * @param validationStringency If STRICT, throw a RuntimeException.  If LENIENT, print the validation
+     * @param validationStringency If STRICT, throw a SAMFormatException.  If LENIENT, print the validation
      * errors to stderr.  If SILENT, do nothing.
      */
     static void processValidationErrors(final List<SAMValidationError> validationErrors,
@@ -331,7 +445,7 @@ public final class SAMUtils
                 validationError.setRecordNumber(samRecordIndex);
             }
             if (validationStringency == SAMFileReader.ValidationStringency.STRICT) {
-                throw new RuntimeException("SAM validation error: " + validationErrors.get(0));
+                throw new SAMFormatException("SAM validation error: " + validationErrors.get(0));
             }
             else if (validationStringency == SAMFileReader.ValidationStringency.LENIENT) {
                 for (final SAMValidationError error : validationErrors) {
@@ -344,7 +458,7 @@ public final class SAMUtils
     static void processValidationError(final SAMValidationError validationError,
                                        SAMFileReader.ValidationStringency validationStringency) {
         if (validationStringency == SAMFileReader.ValidationStringency.STRICT) {
-            throw new RuntimeException("SAM validation error: " + validationError);
+            throw new SAMFormatException("SAM validation error: " + validationError);
         }
         else if (validationStringency == SAMFileReader.ValidationStringency.LENIENT) {
             System.err.println("Ignoring SAM validation error: " + validationError);
@@ -384,4 +498,67 @@ public final class SAMUtils
             }
         }
     }
+
+    public static void makeReadUnmapped(SAMRecord rec) {
+        if (rec.getReadNegativeStrandFlag()) {
+            SAMRecordUtil.reverseComplement(rec);
+            rec.setReadNegativeStrandFlag(false);
+        }
+        rec.setDuplicateReadFlag(false);
+        rec.setReferenceIndex(SAMRecord.NO_ALIGNMENT_REFERENCE_INDEX);
+        rec.setAlignmentStart(SAMRecord.NO_ALIGNMENT_START);
+        rec.setCigarString(SAMRecord.NO_ALIGNMENT_CIGAR);
+        rec.setMappingQuality(SAMRecord.NO_MAPPING_QUALITY);
+        rec.setInferredInsertSize(0);
+        rec.setNotPrimaryAlignmentFlag(false);
+        rec.setProperPairFlag(false);
+        rec.setReadUnmappedFlag(true);
+    }
+
+
+    /**
+     * Determines if a cigar has any element that both consumes read bases and consumes reference bases
+     * (e.g. is not all soft-clipped)
+     */
+    public static boolean cigarMapsNoBasesToRef(final Cigar cigar) {
+        for (final CigarElement el : cigar.getCigarElements()) {
+            if (el.getOperator().consumesReadBases() && el.getOperator().consumesReferenceBases()) {
+                return false;
+            }
+        }
+        return true;
+    }
+
+    /**
+     *
+     * @return negative if mapq1 < mapq2, etc.
+     * Note that MAPQ(0) < MAPQ(255) < MAPQ(1)
+     */
+    public static int compareMapqs(final int mapq1, final int mapq2) {
+        if (mapq1 == mapq2) return 0;
+        if (mapq1 == 0)  return -1;
+        else if (mapq2 == 0) return 1;
+        else if (mapq1 == 255) return -1;
+        else if (mapq2 == 255) return 1;
+        else return mapq1 - mapq2;
+    }
+
+
+    /**
+     * Hokey algorithm for combining two MAPQs into values that are comparable, being cognizant of the fact
+     * that in MAPQ world, 1 > 255 > 0. In this algorithm, 255 is treated as if it were 0.01, so that
+     * CombinedMapq(1,0) > CombinedMapq(255, 255) > CombinedMapq(0, 0).
+     * The return value should not be used for anything other than comparing to the return value of other
+     * invocations of this method.
+     */
+    public static int combineMapqs(int m1, int m2) {
+        if (m1 == 255) m1 = 1;
+        else m1 *= 100;
+
+        if (m2 == 255) m2 = 1;
+        else m2 *= 100;
+
+        return m1 + m2;
+
+    }
 }
