@@ -35,6 +35,9 @@ public class SAMValidationError {
     }
 
     public enum Type {
+        /** quality encodings out of range; appear to be Solexa or Illumina when Phread expected */
+        INVALID_QUALITY_FORMAT(Severity.WARNING),
+        
         /** proper pair flag set for unpaired read */
         INVALID_FLAG_PROPER_PAIR,
 
@@ -77,8 +80,8 @@ public class SAMValidationError {
         /** CIGAR string is empty for mapped read or not empty of unmapped read, or other CIGAR badness. */
         INVALID_CIGAR,
 
-        /** CIGAR string contains I followed by D, or vice versa, with no intervening M */
-        ADJACENCT_INDEL_IN_CIGAR(Severity.WARNING),
+        /** CIGAR string contains I followed by D, or vice versa */
+        ADJACENT_INDEL_IN_CIGAR(Severity.WARNING),
 
         /** mate reference index (MRNM) set for unpaired read */    
         INVALID_MATE_REF_INDEX,
@@ -119,6 +122,9 @@ public class SAMValidationError {
         /** A read group ID on a SAMRecord is not found in the header */
         READ_GROUP_NOT_FOUND,
 
+        /** A SAMRecord is found with no read group id */
+        RECORD_MISSING_READ_GROUP(Severity.WARNING),
+
         /** Indexing bin set on SAMRecord does not agree with computed value. */
         INVALID_INDEXING_BIN,
 
@@ -177,7 +183,10 @@ public class SAMValidationError {
         DUPLICATE_PROGRAM_GROUP_ID,
 
         /** Read is marked as paired, but its pair was not found.  */
-        MATE_NOT_FOUND;
+        MATE_NOT_FOUND,
+
+        /** Both mates are marked as first of pair, or both mates are marked as second of pair. */
+        MATES_ARE_SAME_END;
 
         public final Severity severity;
 
