@@ -31,6 +31,7 @@ import net.sf.picard.fastq.FastqRecord;
 import net.sf.picard.io.IoUtil;
 import net.sf.samtools.SAMFileReader;
 import net.sf.samtools.SAMRecord;
+import net.sf.samtools.util.IOUtil;
 import org.testng.Assert;
 import org.testng.annotations.DataProvider;
 import org.testng.annotations.Test;
@@ -49,9 +50,10 @@ public class SamToFastqTest {
     @DataProvider(name = "okFiles")
     public Object[][] okFiles() {
         return new Object[][] {
-            {"ok/sorted-pair.sam"}, // 5 sorted pairs (10 records) - mate1, mate2
-            {"ok/last-pair-mates-flipped.sam" }, // 4 pairs, :05 mate2, mate1
-            {"ok/first-mate-bof-last-mate-eof.sam"}, // :01 mate1, 4 pairs, :01 mate2
+                {"ok/sorted-pair.sam"}, // 5 sorted pairs (10 records) - mate1, mate2
+                {"ok/sorted-pair-no-rg.sam"}, // 5 sorted pairs (10 records) - mate1, mate2, no read group
+                {"ok/last-pair-mates-flipped.sam" }, // 4 pairs, :05 mate2, mate1
+                {"ok/first-mate-bof-last-mate-eof.sam"}, // :01 mate1, 4 pairs, :01 mate2
         };
     }
 
@@ -152,6 +154,34 @@ public class SamToFastqTest {
         }
     }
 
+    @Test(dataProvider =  "okFiles")
+    public void testOkInterleavedFile(final String samFilename) throws IOException {
+        final File samFile = new File(TEST_DATA_DIR,samFilename);
+        final File pairFile = newTempFastqFile("pair");
+
+        convertFile(new String[]{
+                "INPUT=" + samFile.getAbsolutePath(),
+                "FASTQ=" + pairFile.getAbsolutePath(),
+                "INTERLEAVE=true"
+        });
+
+        final Set<String> outputHeaderSet = createFastqReadHeaderSet(pairFile);
+        // Create map of mate pairs from SAM records
+        final Map<String,MatePair> map = createSamMatePairsMap(samFile) ;
+        Assert.assertEquals(map.size() * 2, outputHeaderSet.size());
+
+        // Ensure that each mate of each pair in SAM file is in the correct fastq pair file
+        for (final Map.Entry<String,MatePair> entry : map.entrySet() ) {
+            final MatePair mpair = entry.getValue();
+            Assert.assertNotNull(mpair.mate1); // ensure we have two mates
+            Assert.assertNotNull(mpair.mate2);
+            Assert.assertEquals(mpair.mate1.getReadName(),mpair.mate2.getReadName());
+            final String readName = mpair.mate1.getReadName() ;
+            Assert.assertTrue(outputHeaderSet.contains(readName+"/1")); // ensure mate is in correct file
+            Assert.assertTrue(outputHeaderSet.contains(readName+"/2"));
+        }
+    }
+
 
     @Test (dataProvider = "badFiles", expectedExceptions= PicardException.class)
     public void testBadFile(final String samFilename) throws IOException {
@@ -189,7 +219,7 @@ public class SamToFastqTest {
         final Map<String, Set<String>> outputSets = new HashMap<String, Set<String>>(groupFiles.length);
 
         SamToFastq samToFq = new SamToFastq();
-        String tmpDir = samToFq.TMP_DIR + "/";
+        String tmpDir = IOUtil.getDefaultTmpDir().getAbsolutePath() + "/";
         String [] args = new String[]{
               "INPUT=" + samFile.getAbsolutePath(),
               "OUTPUT_PER_RG=true",
